/*! 
 *********************************************************************
 *  @file   : LDAP_Class.h
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 19/11/2002
 *
 *  Purpose : Declaration of class LDAP_Class
 *
 *********************************************************************
 */

#if !defined(AFX_LDAP_CLASS_H__9434A69A_84FB_40D5_ADA9_5FCA97B37996__INCLUDED_)
#define AFX_LDAP_CLASS_H__9434A69A_84FB_40D5_ADA9_5FCA97B37996__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ldap.h>

#include "LDAPBrowser_Defines.h"

typedef std::vector< LDAPMod * >						CSTL_ModsVector ;

extern char *LDAP_DefaultAttributes[] ;


class LDAP_Class  
{
	
public:
	LDAP_SERVER_TYPE GetSrvType( void );
	int 				GetSizeLimit();
	void 				SetSizeLimit( int &p_SizeLimit );
	int					Open( LDAP * p_LDAP, LDAP_SERVER_TYPE p_Type );
	LDAPMessage*		GetEntry( long p_Index );
	StringVector		GetAttributeValues( const wxString &p_Name );
	wxString			GetAttributeValue( const wxString &p_Name );
	berval**			GetAttributeBinValues( long p_ResIndex, long p_AttIndex, wxString &p_Name );
	berval*				GetAttributeBinValue( const wxString &p_Name, long p_Index );
	LDAP *				GetLDAPHandle();
	int					Rename( const wxString &p_OldDN, const wxString &p_NewDN, bool p_Move );
	int					Delete( const wxString &p_DN );
	int					WriteMods( const wxString &p_DN, CSTL_ModsVector & p_Mods, bool p_IsNew );
	int					GetAttribute( unsigned long p_ResIndex, unsigned long p_Index, wxString &p_Name, StringVector &p_Values );
	long				GetAttributeCount( unsigned long p_ResNumber );
	wxString			GetDN( unsigned long p_Index );
	unsigned long		GetSize();
	int					Search( const wxString &p_BaseDN, const wxString &p_Filter, int p_GetAttributes, 
						int p_Type, char **p_Attributes = NULL );
	int					Close();
	int					Open( const wxString &p_Server, const wxString & p_Port, const wxString &p_BaseDN , const wxString &p_User, const wxString &p_Password, LDAP_SERVER_TYPE p_Type );
	
	LDAP_Class();
	virtual ~LDAP_Class();


protected:
	LDAP				*m_LDAP;
	bool				m_IsMasterCopy ;
	LDAPMessage			*m_Results;
	LDAP_SERVER_TYPE	m_SrvType ;
private:
	LDAPMessage	*		GotoEntry( long p_Index );
	void 				Free_Results();

	wxString			m_BaseDN ;
	wxString			m_Filter ;
	int					m_Type;
	int					m_GetAttributes;
	
};

#endif // !defined(AFX_LDAP_CLASS_H__9434A69A_84FB_40D5_ADA9_5FCA97B37996__INCLUDED_)
