// LDIF_File.h: interface for the LDIF_File class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LDIF_FILE_H__DBE2FA9C_EB10_450A_BD6B_38BA45653283__INCLUDED_)
#define AFX_LDIF_FILE_H__DBE2FA9C_EB10_450A_BD6B_38BA45653283__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <map>

#include "LDAP_Class.h"
#include "DynamicBuffer.h"

#define		LDIF_NO_ERROR		0
#define		LDIF_EOF			-1
#define		LDIF_BAD_FORMAT		-2
#define		LDIF_BAD_VERSION	-3
#define		LDIF_FILE_ERROR		-4
#define		LDIF_BAD_RECORD		-5
#define		LDIF_BAD_LINE		-6
#define		LDIF_MEMORY_ERROR	-7

#define		LDIF_CR				( char )0x0D
#define		LDIF_LF				( char )0x0A
#define		LDIF_SPACE			( char )0x20
#define		LDIF_POUND			'#'

#define		LDIF_BUFFER_SIZE	4000
#define		LDIF_INCREMENT_SIZE	20000


typedef		std::vector< char * >							LDIF_StringVector	;
typedef		std::vector< berval * >							LDIF_BervalVector	;

typedef		std::map< wxString, LDIF_StringVector >		LDIF_RecordAttributes	;
typedef		std::map< wxString, LDIF_BervalVector >		LDIF_RecordBervalAttributes ;

#define		LDAP_OP_MOD			1
#define		LDAP_OP_ADD			2
#define		LDAP_OP_DEL			3
#define		LDAP_OP_REN			4

class LDIF_Record
{
public:
	void AddBinValue( const wxString & p_Name, const DynamicBuffer &p_Value );
	void AddValue( const wxString & p_Name, const wxString &p_Value );
	void Free_LDAPMod_Array( LDAPMod **p_Mods );
	void Init();
	LDIF_Record();
	~LDIF_Record();

	int							m_Mod_Op;
	int							m_Op ;
	wxString					m_Dn;
	LDIF_RecordAttributes		m_Attributes;
	LDIF_RecordBervalAttributes m_BerAttributes;
	LDAPMod **					Get_LDAPMod_Array();
};

class LDIF_File : public LDAP_Class
{
public:
	int			CreateFile( const wxString & p_FileName );

	//int			Write( LDAP_Class *p_Class );
	//int			Read( LDAP_Class *p_Class );
	int			Import( void );
	int			Export( void );
	int			Close();
	int			OpenFile( const wxString & p_FileName );
	static bool Base64Decode( const DynamicBuffer &p_Input, bool & p_IsBinary, 
							 DynamicBuffer &p_Ret );

	static bool Base64Encode( DynamicBuffer &p_Ret , const unsigned char *p_Buffer
							 , unsigned long p_Size );
	/*
	static bool Base64Encode( wxString &p_Ret , const unsigned char *p_Buffer
							 , unsigned long p_Size );
	//static bool	Base64Encode( wxString &p_Ret , const ByteArray & p_Buffer );
	
	static bool Base64Decode( const wxString &p_Str, unsigned long &p_Size, 
								  bool & p_IsBinary, ByteArray &p_Ret  );
	*/
	/* 
	static bool Base64Decode( const ByteArray & p_Str, unsigned long & p_Size, 
		bool & p_IsBinary, ByteArray & p_Ret  );
	*/
	LDIF_File();
	virtual ~LDIF_File();

private:
	FILE			*m_File ;
	/*
	* The current Line : as non UTF-8 chars are Base64 encoded
	* no binary buffer is needed.
	*/
	//wxString		m_Line ; 

	DynamicBuffer	m_Line ;
	LDIF_Record		m_CurrentRecord;
protected:
	int ProcessRecord();
	int ReadNextLine();
//	bool IsAscii( const wxString & p_Str );
	int WriteRecord( long p_Index );
	int WriteHeader();
	int ReadHeader();
	int ReadLine();
	bool Must_Encode_Dn( const wxString &p_DN );

};

#endif // !defined(AFX_LDIF_FILE_H__DBE2FA9C_EB10_450A_BD6B_38BA45653283__INCLUDED_)
