/*! 
 *********************************************************************
 *  @file   : DynamicBuffer.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 17/03/2003
 *
 *  Purpose : Implementation of methods for class DynamicBuffer
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include "../include/LDAPBrowser_Defines.h"
#include "../include/DynamicBuffer.h"


DynamicBuffer::DynamicBuffer( unsigned long p_Size, unsigned long p_Increment )
{
	m_Increment = p_Increment;
	m_Size = p_Size ;
	m_Buffer = new unsigned char[ m_Size ];
	Reset();
}
DynamicBuffer::DynamicBuffer( const DynamicBuffer & p_Copy )
{
	if( this != &p_Copy )
	{
		m_Size = p_Copy.m_Size ;
		m_Increment = p_Copy.m_Increment;
		
		m_Buffer = new unsigned char[ m_Size ];
		Reset();
		if( m_Buffer != NULL )
		{
			memcpy( m_Buffer, p_Copy.m_Buffer, p_Copy.m_Pos );
			m_Pos = p_Copy.m_Pos ;
		}
	}
}
DynamicBuffer::DynamicBuffer( const wxString & p_Str )
{
	if( p_Str.length() > DYN_BUFFER_SIZE )
	{
		m_Size = p_Str.length() + 1 ;
	}
	else
	{
		m_Size = DYN_BUFFER_SIZE;
	}

	m_Increment = DYN_BUFFER_INCREMENT;
		
	m_Buffer = new unsigned char[ m_Size ];
	Reset();
	if( m_Buffer != NULL )
	{
		memcpy( m_Buffer, p_Str.c_str(), p_Str.length() );
		m_Pos = p_Str.length() ;
	}

}
const DynamicBuffer & DynamicBuffer::operator=( const DynamicBuffer & p_Copy )
{
	if( this != &p_Copy )
	{
		m_Size = p_Copy.m_Size ;
		m_Increment = p_Copy.m_Increment;
		
		m_Buffer = new unsigned char[ m_Size ];
		Reset();
		if( m_Buffer != NULL )
		{
			memcpy( m_Buffer, p_Copy.m_Buffer, p_Copy.m_Pos );
			m_Pos = p_Copy.m_Pos ;
		}
	}
	return *this;
}
DynamicBuffer::~DynamicBuffer()
{
	if( m_Buffer != NULL )
	{
		delete []m_Buffer ;
		m_Buffer = NULL ;
	}
	m_Size = 0;
}

const char * DynamicBuffer::c_str() const
{
	return ( const char * )m_Buffer ;
}
/*
char DynamicBuffer::operator[]( unsigned long p_Pos )
{
	if( p_Pos <= m_Size )
	{
		return m_Buffer[ p_Pos ];
	}
	return 0;
}
*/
unsigned char DynamicBuffer::operator[]( unsigned long p_Pos ) const 
{
	if( p_Pos <= m_Size )
	{
		return m_Buffer[ p_Pos ];
	}
	return 0;
}
void DynamicBuffer::Reset()
{
	if( m_Buffer != NULL )
	{
		memset( m_Buffer, 0, m_Size );
	}
	m_Pos = 0;
}

unsigned long DynamicBuffer::length() const 
{
	return m_Pos ;
}

void DynamicBuffer::append(char c)
{
	bool bRet = true;
	if( m_Pos >= m_Size )
	{
		bRet = increase();
	}
	if( bRet )
	{
		m_Buffer[ m_Pos ] = c ;
		m_Pos ++ ;
	}
	
}

bool DynamicBuffer::increase()
{
	unsigned char *NewBuf ;

	NewBuf = new unsigned char[ m_Size + m_Increment ];
	if( NewBuf == NULL )
	{
		return false;
	}
	memset( NewBuf, 0, m_Size + m_Increment );
	memcpy( NewBuf, m_Buffer, m_Pos );
	delete []m_Buffer ;
	m_Buffer = NewBuf;
	m_Size = m_Size + m_Increment;
	return true;
}


unsigned long DynamicBuffer::size() const 
{
	return m_Pos;
}

unsigned long DynamicBuffer::find(unsigned char p_Char) const
{
	unsigned long	i = 0 ;
	bool			Found = false;
	while( !Found && i < m_Pos )
	{
		if( m_Buffer[ i ] == p_Char )
		{
			Found = true;
		}
		else
		{
			i++;
		}
	}
	if( Found )
	{
		return i;
	}
	else
	{
		return npos();
	}
}

unsigned long DynamicBuffer::npos() const
{
	return m_Pos + 1;
}

wxString DynamicBuffer::substr(unsigned long p_Pos1, unsigned long p_Pos2) const
{
	wxString Ret( ( const char * )m_Buffer + p_Pos1, *wxConvCurrent, p_Pos2 - p_Pos1 );
	return Ret;
}
wxString DynamicBuffer::substr( unsigned long p_Pos1 ) const
{
	wxString Ret( ( const char * )m_Buffer + p_Pos1, *wxConvCurrent, m_Pos - p_Pos1 );
	return Ret;
}
