/*! 
 *********************************************************************
 *  @file   : LDAPBrowser_App.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 10/11/2002
 *
 *  Purpose : Implementation of methods for class LDAPBrowser_App
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <wx/app.h>
#include <wx/resource.h>
#include <wx/config.h>
#include <wx/intl.h>
#include <wx/artprov.h>
#include <wx/image.h>
#include <wx/xrc/xmlres.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "../include/LDAPBrowser_Defines.h"

#include "../include/LDAPBrowser_MainFrame.h"
#include "../include/LB_ArtProvider.h"
#include "../include/LDAPBrowser_App.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LDAPBrowser_App::LDAPBrowser_App()
{
	m_CurrentFrameID = 0;
}

LDAPBrowser_App::~LDAPBrowser_App()
{

}


IMPLEMENT_APP( LDAPBrowser_App )

bool LDAPBrowser_App::OnInit()
{
	//wxConfigBase	*Conf;
	wxConfig		*Conf;
	int				Language ;
	wxString		XRCDir, DefaultXRCDir ;
	wxString		Art, DefaultArt ;

	wxInitAllImageHandlers();

	SetVendorName( wxEmptyString );
	SetAppName( wxT(PACKAGE) );

	
	//Conf = wxConfigBase::Get();
	Conf = new wxConfig( wxT(PACKAGE) );
	
	
	// Write the default values if they're not present,
	// so the config really shows the values we're using.
	Conf->SetRecordDefaults( TRUE );
	
	
	// First, let's figure out some default values
#ifdef DEFAULT_WXRCDIR
	DefaultXRCDir = DEFAULT_WXRCDIR ;
#else
	DefaultXRCDir = wxT(PACKAGE_DATA_DIR);
#endif

#ifdef DEFAULT_ART
	DefaultArt = DEFAULT_ART ;
#else
	DefaultArt = wxT(PACKAGE_DATA_DIR);
#endif

	Language = Conf->Read( wxT("Lang"), wxLANGUAGE_DEFAULT );
	XRCDir = Conf->Read( wxT("XRCDirectory"), DefaultXRCDir );
	Art = Conf->Read( wxT("Art"), DefaultArt );
	Conf->Flush();

	delete Conf ;

#ifndef __BORLANDC__		
	m_Locale.Init( Language ); 
	m_Locale.AddCatalog(wxT(PACKAGE)); 
#endif

	// All the .xrc files in the XRC directory will be loaded
	wxString AllXRC = XRCDir + wxT("/*.xrc");
	wxXmlResource::Get()->InitAllHandlers();
	wxXmlResource::Get()->Load( AllXRC );
	wxXmlResource::Get()->SetFlags( wxXRC_USE_LOCALE );

	LB_ArtProvider *MyProvider = new LB_ArtProvider();

	wxArtProvider::PushProvider( MyProvider );
	LDAPBrowser_MainFrame * NewFrame = NULL ;
	long					FrameID	= -1;

	FrameID = GetFrameID();

	NewFrame = new LDAPBrowser_MainFrame( NULL , FrameID , wxT(_PROJECT_NAME) );
	if( NewFrame == NULL )
	{
		return false;
	}

	m_Frames[ FrameID ] = NewFrame ;
	NewFrame->Centre( wxBOTH );
	NewFrame->Show( TRUE );

	return TRUE;

}

int LDAPBrowser_App::OnExit()
{
	return 0;
}


long LDAPBrowser_App::GetFrameID()
{
	m_CurrentFrameID ++ ;
	return m_CurrentFrameID;
}

void LDAPBrowser_App::RemoveWindow( long p_ID )
{
	m_Frames.erase( p_ID );
}

void LDAPBrowser_App::OnNew( wxMenuEvent &p_Evt )
{

	LDAPBrowser_MainFrame * NewFrame = NULL ;
	long					FrameID	= -1;
	
	FrameID = GetFrameID();

	NewFrame = new LDAPBrowser_MainFrame( NULL , FrameID , wxT(_PROJECT_NAME) );
	if( NewFrame == NULL )
	{
		return ;
	}

	m_Frames[ FrameID ] = NewFrame ;
	NewFrame->Show( TRUE );

}
