/*! 
 *********************************************************************
 *  @file   : LDAPBrowser_CreateObjectDialog.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 19/11/2002
 *
 *  Purpose : Implementation of methods for class LDAPBrowser_CreateObjectDialog
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <algorithm>

#include <wx/dialog.h>
#include <wx/choice.h>
#include <wx/textctrl.h>
#include <wx/listbox.h>
#include <wx/intl.h>
#include <wx/xrc/xmlres.h>

#include "../include/LDAPBrowser_Defines.h"
#include "../include/LDAP_Class.h"
#include "../include/LDAP_Schema.h"
#include "../include/LDAPBrowser_CreateObjectDialog.h"

BEGIN_EVENT_TABLE(LDAPBrowser_CreateObjectDialog, wxDialog)
	EVT_BUTTON( XRCID( "button_create_object_cancel" ), LDAPBrowser_CreateObjectDialog::OnCancel )
	EVT_BUTTON( XRCID( "button_create_object_ok" ), LDAPBrowser_CreateObjectDialog::OnOk )
	EVT_CHOICE( XRCID( "create_object_objectclass" ), LDAPBrowser_CreateObjectDialog::OnItemSelected )
END_EVENT_TABLE()


LDAPBrowser_CreateObjectDialog::LDAPBrowser_CreateObjectDialog( wxWindow* p_Parent )
{
	wxXmlResource::Get()->LoadDialog( this, p_Parent, wxT("create_object_dialog") );
	SetSize( wxSize( 300 , 300 ) );
	m_Schema = NULL;

	m_NewDN = ( wxTextCtrl * )FindWindowByName( wxT("create_object_dn"), this );
	m_RDN = ( wxTextCtrl * )FindWindowByName( wxT("create_object_relativedn"), this );

	m_ObjectClass = ( wxChoice * )FindWindowByName( wxT("create_object_objectclass"), this );
	m_Mandatories = ( wxListBox * )FindWindowByName( wxT("create_object_mandatory"), this );
	m_Optionals = ( wxListBox * )FindWindowByName( wxT("create_object_optional"), this );
}

LDAPBrowser_CreateObjectDialog::~LDAPBrowser_CreateObjectDialog()
{

}

void LDAPBrowser_CreateObjectDialog::OnOk( wxCommandEvent &event )
{
	WindowToMembers();
	if( m_StrDN.Length() > 0 )
	{
		EndModal( wxID_OK );
	}
}

void LDAPBrowser_CreateObjectDialog::OnCancel( wxCommandEvent &event )
{
	EndModal( wxID_CANCEL );

}

void LDAPBrowser_CreateObjectDialog::SetDN( const wxString &p_DN )
{
	/*
	* Should set the DN text field to p_DN ;
	*/
	m_StrDN = p_DN;
	if( m_NewDN != NULL )
	{
		m_NewDN->SetValue( m_StrDN );
	}
	/*
	m_FirstPanel->TransferDataToWindow();
	*/
}

wxString LDAPBrowser_CreateObjectDialog::GetDN()
{
	/*
	m_FirstPanel->TransferDataFromWindow();
	*/
	if( m_NewDN != NULL )
	{
		m_StrDN = m_NewDN->GetValue();
	}
	return m_StrDN;
}


void LDAPBrowser_CreateObjectDialog::SetRDN( const wxString &p_RDN )
{
	/*
	* Should set the relative DN text field to p_DN ;
	*/
	m_StrRDN = p_RDN ;

	if( m_RDN != NULL )
	{
		m_RDN->SetValue( m_StrRDN );
	}
}

void LDAPBrowser_CreateObjectDialog::SetSchema( LDAP_Schema *p_Schema )
{
	long nb,i;
	m_Schema = p_Schema ;
	

	StringVector ObjectClasses;


	ObjectClasses = m_Schema->GetObjectClasses();

	std::sort( ObjectClasses.begin(), ObjectClasses.end() );

	nb = ObjectClasses.size();

	if( m_ObjectClass != NULL )
	{
		for( i = 0 ; i < nb ; i ++)
		{
			m_ObjectClass->Append( ObjectClasses[ i ].c_str() );
		}
	}
}

void LDAPBrowser_CreateObjectDialog::OnItemSelected( wxEvent &p_Evt )
{
	wxString	Selection; 
	long		nb, i;
	StringVector Attributes;

	if( m_Schema == NULL )
	{
		return;
	}

	if( m_ObjectClass != NULL && m_Mandatories != NULL && m_Optionals != NULL )
	{
		Selection = m_ObjectClass->GetStringSelection();
		if( Selection.Length() > 0 )
		{
			m_Mandatories->Clear();
			m_Optionals->Clear();

			Attributes = m_Schema->GetObjectClassAttributes( Selection.c_str(), ATTRIBUTE_TYPE_MUST ) ;
			nb = Attributes.size();
			for( i = 0 ; i < nb ; i ++ )
			{
				m_Mandatories->Append( Attributes[ i ].c_str() );
			}
			Attributes = m_Schema->GetObjectClassAttributes( Selection.c_str(), ATTRIBUTE_TYPE_MAY ) ;
			nb = Attributes.size();
			for( i = 0 ; i < nb ; i ++ )
			{
				m_Optionals->Append( Attributes[ i ].c_str() );
			}
		}
	}
}

void LDAPBrowser_CreateObjectDialog::WindowToMembers()
{
	wxArrayInt		Selected;
	wxString		Selection; 
	long			nb, i;

	
	if( m_NewDN != NULL )
	{
		m_StrDN = m_NewDN->GetValue();
	}
	if( m_RDN != NULL )
	{
		m_StrRDN = m_RDN->GetValue();
	}

	StringVector Attributes;

	if( m_Schema != NULL )
	{
		if( m_ObjectClass != NULL )
		{
			m_ObjectClassStr= m_ObjectClass->GetStringSelection();
			if( m_ObjectClassStr.length() > 0 )
			{
				m_MandatoriesList = m_Schema->GetObjectClassAttributes( m_ObjectClassStr, ATTRIBUTE_TYPE_MUST ) ;
			}
		}
		if( m_Optionals != NULL )
		{
			nb = m_Optionals->GetSelections( Selected );
			m_OptionalsList.clear();
			
			for( i = 0 ; i < nb ; i++ )
			{
				m_OptionalsList.push_back( m_Optionals->GetString( Selected[i] ).c_str() );
			}
		}
	}
	
}
