/*! 
 *********************************************************************
 *  @file   : LDAPBrowser_LeftPane.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 10/11/2002
 *
 *  Purpose : Implementation of methods for class LDAPBrowser_LeftPane
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <wx/treectrl.h>
#include <wx/imaglist.h>
#include <wx/menu.h>
#include <wx/textdlg.h>
#include <wx/intl.h>
#include <wx/msgdlg.h>
#include <wx/xrc/xmlres.h>

#include "../include/LDAPBrowser_Defines.h"
#include "../include/LDAPBrowser_MainFrame.h"
#include "../include/LDAPBrowser_LeftPane.h"
#include "../include/LB_ArtProvider.h"

BEGIN_EVENT_TABLE( LDAPBrowser_LeftPane , wxTreeCtrl )
	EVT_TREE_ITEM_COLLAPSED( -1 , LDAPBrowser_LeftPane::OnItemCollapsed )
	EVT_TREE_ITEM_EXPANDED( -1 , LDAPBrowser_LeftPane::OnItemExpanded ) 
	EVT_TREE_ITEM_ACTIVATED( -1 , LDAPBrowser_LeftPane::OnItemActivated )
	EVT_TREE_SEL_CHANGED( -1 , LDAPBrowser_LeftPane::OnSelectionChanged )

	EVT_RIGHT_DOWN( LDAPBrowser_LeftPane::OnRightMouseClick )

	EVT_MENU( XRCID( "left_popup_menu_moddn" ), LDAPBrowser_LeftPane::OnPopupModify )
	EVT_MENU( XRCID( "left_popup_menu_movdn" ), LDAPBrowser_LeftPane::OnPopupMove )
	EVT_MENU( XRCID( "left_popup_menu_addentry" ), LDAPBrowser_LeftPane::OnPopupAdd )
	EVT_MENU( XRCID( "left_popup_menu_delentry" ), LDAPBrowser_LeftPane::OnPopupDelete )
	EVT_MENU( XRCID( "left_popup_menu_refresh" ), LDAPBrowser_LeftPane::OnPopupRefresh )
	EVT_MENU( XRCID( "left_popup_menu_search" ), LDAPBrowser_LeftPane::OnSearch )

END_EVENT_TABLE()

LDAPBrowser_LeftPane::LDAPBrowser_LeftPane(wxWindow* parent, wxWindowID id, const wxPoint& pos , const wxSize& size , long style ,const wxValidator& validator , const wxString& name )
:wxTreeCtrl(parent,id,pos,size,style,validator,name)
{
	m_IsSearchResult = 0 ;
	m_TreeStdImageList = NULL ;
	m_TreeStateImageList = NULL ;

	m_TreeStdImageList = new wxImageList( 16, 16 );
	m_TreeStateImageList = new wxImageList( 16, 16 );

	wxBitmap	Bmp1 = wxArtProvider::GetBitmap( wxART_LB_OPENED_FOLDER );
	wxBitmap	Bmp2 = wxArtProvider::GetBitmap( wxART_LB_CLOSED_FOLDER );

	m_TreeStdImageList->Add( Bmp2 );
	m_TreeStdImageList->Add( Bmp1 );

	m_TreeStateImageList->Add( Bmp2 );
	m_TreeStateImageList->Add( Bmp1 );

	SetImageList( m_TreeStdImageList );
	SetStateImageList( m_TreeStateImageList );
}

LDAPBrowser_LeftPane::~LDAPBrowser_LeftPane()
{
	if( m_TreeStdImageList != NULL )
		delete m_TreeStdImageList ;
	if( m_TreeStateImageList != NULL )
		delete m_TreeStateImageList ;

}

void LDAPBrowser_LeftPane::OnItemCollapsed( wxTreeEvent &p_Event )
{
}

void LDAPBrowser_LeftPane::OnItemExpanded( wxTreeEvent &p_Event )
{
}

void LDAPBrowser_LeftPane::OnItemActivated( wxTreeEvent &p_Evt )
{
	wxTreeItemId	Id ;

	Id = p_Evt.GetItem();
	LDAPBrowser_MainFrame *Parent ;
	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();

	Parent->OnItemActivated( Id );
}

void LDAPBrowser_LeftPane::OnRightMouseClick( wxMouseEvent &p_Evt )
{
	wxMenu	*Popup = NULL ;
	wxPoint	Position;
	wxTreeItemId CurrentSelectedItem ;

	Position = p_Evt.GetPosition();
	CurrentSelectedItem = HitTest( Position );

	SelectItem( CurrentSelectedItem );

	if( CurrentSelectedItem.IsOk() )
	{
		Popup = wxXmlResource::Get()->LoadMenu( wxT("left_popup_menu") );
		PopupMenu( Popup , Position ) ;
	}
	delete Popup;

}

void LDAPBrowser_LeftPane::OnPopupModify( wxMenuEvent &p_Evt )
{
	wxTreeItemId		Id;
	int					Ret;
	wxString			Dn;
	
	Id = GetSelection() ;
	while ( Id != GetRootItem() )
	{
		Dn = Dn + GetItemText( Id );
		Id = GetParent( Id );
		Dn = Dn + wxT(",");
	}
	Dn += GetItemText( Id );

	wxTextEntryDialog	Dlg( this , _( "Enter new DN of the entry" ) , _( "Entry modification" ) , Dn );

	Dlg.SetValue( Dn );
	Ret = Dlg.ShowModal();

	if( Ret == wxID_OK )
	{
		LDAPBrowser_MainFrame *Parent ;
		Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();
		
		int iRet = Parent->m_LDAP.Rename( Dn, Dlg.GetValue(), false );
		if( iRet == LDAP_SUCCESS )
		{
			// Move up the selection one level
			SelectItem( GetParent( GetSelection() ) ) ;
			OnPopupRefresh( p_Evt );
		}
		else
		{
			wxString ErrMsg ( ldap_err2string( iRet ), *wxConvCurrent );
			wxMessageDialog Dlg( this , ErrMsg, _( "LDAP Error" ) , wxOK );
			Dlg.ShowModal();
		}

	}
}

void LDAPBrowser_LeftPane::OnPopupMove( wxMenuEvent &p_Evt )
{
	wxTreeItemId		Id;
	int					Ret;
	wxString			Txt;
	wxString			Dn;
	
	Id = GetSelection() ;
	Txt = GetItemText( Id );
	while ( Id != GetRootItem() )
	{
		Dn = Dn + GetItemText( Id );
		Id = GetParent( Id );
		Dn = Dn + wxT(",");
	}
	Dn += GetItemText( Id );

	wxTextEntryDialog	Dlg( this , _( "Enter new DN of the entry" ) , _( "Entry modification" ) , Dn );

	Dlg.SetValue( Dn );
	Ret = Dlg.ShowModal();

	if( Ret == wxID_OK )
	{
		LDAPBrowser_MainFrame *Parent ;
		Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();

		int iRet = Parent->m_LDAP.Rename( Dn, Dlg.GetValue(), true );
		if( iRet == LDAP_SUCCESS )
		{
			// Move up the selection one level
			SelectItem( GetParent( GetSelection() ) ) ;
			OnPopupRefresh( p_Evt );
		}
		else
		{
			wxString ErrMsg ( ldap_err2string( iRet ), *wxConvCurrent );
			wxMessageDialog Dlg( this , ErrMsg, _( "LDAP Error" ) , wxOK );
			Dlg.ShowModal();
		}

	}
}

void LDAPBrowser_LeftPane::OnPopupAdd( wxMenuEvent &p_Evt )
{
	LDAPBrowser_MainFrame *Parent ;
	wxTreeItemId CurrentSelectedItem ;
	
	CurrentSelectedItem = GetSelection() ;
	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();
	Parent->OnNewEntry( CurrentSelectedItem );
}

void LDAPBrowser_LeftPane::OnPopupDelete( wxMenuEvent &p_Evt )
{
	LDAPBrowser_MainFrame *Parent ;
	wxTreeItemId CurrentSelectedItem ;
	
	CurrentSelectedItem = GetSelection() ;
	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();

	wxMessageDialog		Dlg( this , _( "Do you really want to delete this entry ?" ) , _( "Delete Entry" ) , wxYES_NO | wxNO_DEFAULT  ) ;
	
	if( Dlg.ShowModal() == wxID_NO )
	{
		return ;
	}
	int iRet = Parent->OnDeleteEntry( CurrentSelectedItem );
	if( iRet != LDAP_SUCCESS )
	{
		wxString ErrMsg ( ldap_err2string( iRet ), *wxConvCurrent );
		wxMessageDialog Dlg( this , ErrMsg, _( "LDAP Error" ), wxOK );
		Dlg.ShowModal();
	}
	else
	{
		Delete( CurrentSelectedItem );
	}
}

void LDAPBrowser_LeftPane::OnPopupRefresh( wxMenuEvent &p_Evt )
{
	wxTreeItemId CurrentSelectedItem ;
	
	CurrentSelectedItem = GetSelection() ;

	// Delete all the children, they will be recreated 
	// by Parent->OnItemActivated( )
	DeleteChildren( CurrentSelectedItem );

	LDAPBrowser_MainFrame *Parent ;
	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();
	
	Parent->OnItemActivated( CurrentSelectedItem );
}

void LDAPBrowser_LeftPane::OnSelectionChanged( wxTreeEvent &p_Evt )
{
	wxTreeItemId CurrentSelectedItem ;
	
	CurrentSelectedItem = p_Evt.GetItem();
	if( CurrentSelectedItem.IsOk() )
	{
		LDAPBrowser_MainFrame *Parent ;
		Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();

		Parent->OnItemSelected( CurrentSelectedItem );
	}
}

void LDAPBrowser_LeftPane::OnSearch( wxMenuEvent &p_Evt )
{
	wxTextEntryDialog	Dlg( this , _( "Enter the search criteria" ) , _( "Search" ) , wxT("") );
	wxTreeItemId		Id = GetSelection();
	int					Ret;

	if( !Id.IsOk() || ( m_IsSearchResult && Id == GetRootItem() ) )
		return;
	Ret = Dlg.ShowModal();

	if( Ret == wxID_OK )
	{
		LDAPBrowser_MainFrame *Parent ;
		Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();
		
		Parent->OnSearch( Id , Dlg.GetValue() );

	}
}
