/*! 
 *********************************************************************
 *  @file   : LDAPBrowser_MainFrame.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 10/11/2002
 *
 *  Purpose : Implementation of methods for class LDAPBrowser_MainFrame
 *
 *********************************************************************
 */
 
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <wx/intl.h>
#include <wx/app.h>
#include <wx/listctrl.h>
#include <wx/msgdlg.h>
#include <wx/filedlg.h>
#include <wx/statusbr.h>
#include <wx/xrc/xmlres.h>
#include <wx/log.h>

#include "../include/LDAPBrowser_Defines.h"
#include "../include/LDAPBrowser_App.h"
#include "../include/LDAPBrowser_LeftPane.h"
#include "../include/LDAPBrowser_RightPane.h"
#include "../include/LDAPBrowser_MainFrame.h"
#include "../include/LDAPBrowser_OpenDialog.h"
#include "../include/LDAPBrowser_CreateObjectDialog.h"
#include "../include/LDAPBrowser_PreferencesDialog.h"
#include "../include/LDAPBrowser_About.h"
#include "../include/LDIF_File.h"


BEGIN_EVENT_TABLE( LDAPBrowser_MainFrame , wxFrame )
    EVT_MENU( XRCID( "file_menu_connect" ), LDAPBrowser_MainFrame::OnOpen )
    EVT_MENU( XRCID( "file_menu_newwindow" ), LDAPBrowser_MainFrame::OnNew )
    EVT_MENU( XRCID( "file_menu_quit" ), LDAPBrowser_MainFrame::Close )

    EVT_MENU( XRCID( "help_menu_about" ), LDAPBrowser_MainFrame::OnAbout )

	EVT_MENU( XRCID( "file_menu_export" ), LDAPBrowser_MainFrame::OnExport )
	EVT_MENU( XRCID( "file_menu_import" ), LDAPBrowser_MainFrame::OnImport )

	EVT_MENU( XRCID( "edit_menu_addentry" ), LDAPBrowser_MainFrame::OnFileAddEntry )
	EVT_MENU( XRCID( "edit_menu_delentry" ), LDAPBrowser_MainFrame::OnFileDelEntry )
	EVT_MENU( XRCID( "edit_menu_moddn" ), LDAPBrowser_MainFrame::OnFileModDNEntry )
	EVT_MENU( XRCID( "edit_menu_movdn" ), LDAPBrowser_MainFrame::OnFileMovDNEntry )
	EVT_MENU( XRCID( "edit_menu_additem" ), LDAPBrowser_MainFrame::OnFileAddAttribute )
	EVT_MENU( XRCID( "edit_menu_delitem" ), LDAPBrowser_MainFrame::OnFileDelAttribute )
	EVT_MENU( XRCID( "edit_menu_moditem" ), LDAPBrowser_MainFrame::OnFileModAttribute )
	EVT_MENU( XRCID( "edit_menu_addbinitem" ), LDAPBrowser_MainFrame::OnFileAddBinAttribute )
	EVT_MENU( XRCID( "edit_menu_delbinitem" ), LDAPBrowser_MainFrame::OnFileDelBinAttribute )
	EVT_MENU( XRCID( "edit_menu_viewbinitem" ), LDAPBrowser_MainFrame::OnFileViewBinAttribute )
	EVT_MENU( XRCID( "edit_menu_exportbinitem" ), LDAPBrowser_MainFrame::OnFileExportBinAttribute )
	EVT_MENU( XRCID( "edit_menu_saveitem" ), LDAPBrowser_MainFrame::OnSave )
	EVT_MENU( XRCID( "edit_menu_preferences" ), LDAPBrowser_MainFrame::OnEditPreferences )

    EVT_TOOL( XRCID( "button_newwindow" ), LDAPBrowser_MainFrame::OnNew )
	EVT_TOOL( XRCID( "button_connect" ) , LDAPBrowser_MainFrame::OnOpen )
END_EVENT_TABLE()



LDAPBrowser_MainFrame::LDAPBrowser_MainFrame( wxWindow * p_Parent , long p_ID , const wxString & p_WindowName ) 
{
	if(!(wxXmlResource::Get()->LoadFrame( this, p_Parent, wxT("main_frame") ) ))
	{
		::wxLogError( _("Failed to find good MainFrame info in global resources object\n") );
		Create(p_Parent, p_ID, p_WindowName);
	}
	m_ToolBar = NULL ;
	m_Splitter = NULL ;
	m_Percent = (float)0.3;
	m_DataLoaded = 0 ;
	m_RightPane = NULL ; 
	m_LeftPane = NULL ;

	SetTitle( p_WindowName );

	SetIcon( wxIcon( wxT("AAAAAAAAA") ) );
	Initialize();
}

LDAPBrowser_MainFrame::~LDAPBrowser_MainFrame()
{
	if( m_RightPane != NULL )
		delete m_RightPane ;
	if( m_LeftPane != NULL )
		delete m_LeftPane ;
	if( m_AboutDialog != NULL )
		delete m_AboutDialog ;
}

int LDAPBrowser_MainFrame::CreateMenuBar()
{
	wxMenuBar *MenuBar = NULL ;

	MenuBar = wxXmlResource::Get()->LoadMenuBar( wxT("main_menu") );
	if( MenuBar != NULL )
	{
		SetMenuBar( MenuBar );
	}
	return 0;
}

int LDAPBrowser_MainFrame::Initialize()
{

	wxString		Status;

	CreateMenuBar();
	wxStatusBar *Sb = CreateStatusBar();

	Status = wxT(PACKAGE);
	Status += wxT(" ");
	Status += wxT(VERSION);
	Sb->SetStatusText( Status );
	wxSize FrameSize = this->GetSize();

	/* Creating the ToolBar */
	DrawToolBar();

	/* Creating the Window */

	m_Splitter = new wxSplitterWindow( this , -1 , wxDefaultPosition , FrameSize );

	m_LeftPane = new LDAPBrowser_LeftPane( m_Splitter , -1 , wxDefaultPosition , wxSize( FrameSize.x / 3 , FrameSize.y ) );
	m_RightPane = new LDAPBrowser_RightPane( m_Splitter , -1 , wxDefaultPosition ,  wxSize( 2*( FrameSize.x / 3 ) , FrameSize.y ) , wxTR_HAS_BUTTONS | wxTR_EDIT_LABELS );

	m_Splitter->SplitVertically( m_LeftPane , m_RightPane );
	m_Splitter->SetMinimumPaneSize( 20 );
	
	m_AboutDialog = new LDAPBrowser_About( this );

	return 0;
}

void LDAPBrowser_MainFrame::OnOpen(wxMenuEvent &event)
{
	LDAPBrowser_OpenDialog			*Dlg= new LDAPBrowser_OpenDialog( this , -1 , _( "Open ..." ) );
	int								iRet;
	wxString						Text;
	
	iRet = Dlg->ShowModal( );
	if( iRet == wxID_CANCEL )
		return ;

	
	iRet = m_LDAP.Open( Dlg->m_StrServer, Dlg->m_StrPort, Dlg->m_StrBaseDN, Dlg->m_StrUser, Dlg->m_StrPassword, Dlg->m_SrvType );
	if( iRet != LDAP_SUCCESS )
	{
		wxString ErrMsg ( ldap_err2string( iRet ), *wxConvCurrent);
		::wxMessageBox( ErrMsg, _( "LDAP Error" ), wxOK | wxCENTRE | wxICON_ERROR );
		return ;
	}

	
	iRet = m_LDAP.Search( Dlg->m_StrBaseDN , wxT("(objectClass=*)") , LDAP_SCOPE_BASE , 0 );
	if( iRet != 0 )
	{
		::wxLogMessage( wxT("LDAP Error : %s \nSearching %s"), ldap_err2string( iRet ), 
			Dlg->m_StrBaseDN.c_str() );
		return ;
	}
	m_Schema.Open( m_LDAP.GetLDAPHandle(), Dlg->m_SrvType );
	m_Schema.OpenSchema();

	wxString Title ;

	Title = wxT(_PROJECT_NAME);
	Title += wxT(" - ") + Dlg->m_StrBaseDN ;
	SetTitle( Title );

	m_LeftPane->m_IsSearchResult = 0 ;
	m_LeftPane->DeleteAllItems();


	wxTreeItemId Id = m_LeftPane->AddRoot( Dlg->m_StrBaseDN , 0 , -1 );
	m_LeftPane->SetItemImage( Id , 0 , wxTreeItemIcon_Normal );
	m_LeftPane->SetItemImage( Id , 0 , wxTreeItemIcon_Selected );
	m_LeftPane->SetItemImage( Id , 1 , wxTreeItemIcon_Expanded );
	m_LeftPane->SetItemImage( Id , 1 , wxTreeItemIcon_SelectedExpanded );

	delete Dlg;

	OnItemActivated( Id );
}

void LDAPBrowser_MainFrame::DrawToolBar()
{
	wxToolBar *ToolBar = NULL ;

	ToolBar = wxXmlResource::Get()->LoadToolBar( this, wxT("main_toolbar") );
	if( ToolBar != NULL )
	{
		SetToolBar( ToolBar );
	}
}





void LDAPBrowser_MainFrame::OnCloseWindow(wxCloseEvent &event)
{
	wxGetApp().RemoveWindow( m_ID ) ;
	m_LDAP.Close();
}



/*!
 * This method is called when an item is clicked or double clicked in the left pane ...
 *
 * @param p_Id : id of the current selected item in the left pane
 *
 * @return void  : 
 */
void LDAPBrowser_MainFrame::OnItemActivated( wxTreeItemId & p_Id )
{

	wxTreeItemId	ChildId;
	long			cookie;
	long			iRet;
	wxString		Dn;
	wxString		ConvertedString;

	/*
	* First check if the right pane's entry was being edited and not already saved
	*/
	if( m_RightPane->IsModified() )
	{
		iRet = ::wxMessageBox( _( "Some changes have not been saved\n( this is new entry or an empty attribute has been added ).\nDo you wish to continue ?" ),
			m_RightPane->GetDN().c_str(), wxYES_NO | wxNO_DEFAULT | wxICON_EXCLAMATION );

		if( iRet == wxYES )
		{
			/*
			* User does not want to save changes 
			*/
			m_RightPane->IsModified( false, true );
			m_RightPane->IsNew( false );
		};
		return ;
	}

	m_RightPane->EmptyPane();
		
	Dn = GetDN( p_Id );		
	
	ChildId = m_LeftPane->GetFirstChild( p_Id , cookie );
	if( !ChildId.IsOk() )
	{
		/*
		* Activated item does not have childs in the tree view
		* Verify if it the corresponding entry in LDAP does
		*/
		
		iRet = m_LDAP.Search( Dn, wxT("objectClass=*") ,1 ,LDAP_SCOPE_ONELEVEL );
		if( iRet == 0 && m_LDAP.GetSize() > 0 )
		{
			for( unsigned long i = 0 ; i < m_LDAP.GetSize() ; i ++ )
			{
				/*
				* the entry has childs add them to the left pane's tree.
				*/
				wxTreeItemId NewItem ;
				char **Values = ldap_explode_dn( m_LDAP.GetDN( i ).mb_str() , 0 );

				wxString Item ( Values[ 0 ], wxConvUTF8 );				
				NewItem = m_LeftPane->AppendItem( p_Id, Item, 0, 0 );
				
				ldap_value_free( Values );

				m_LeftPane->SetItemImage( NewItem , 0 , wxTreeItemIcon_Normal );
				m_LeftPane->SetItemImage( NewItem , 0 , wxTreeItemIcon_Selected );
				m_LeftPane->SetItemImage( NewItem , 1 , wxTreeItemIcon_Expanded );
				m_LeftPane->SetItemImage( NewItem , 1 , wxTreeItemIcon_SelectedExpanded );
			}
			if( p_Id == m_LeftPane->GetRootItem() )
			{
				m_LeftPane->Expand( p_Id );
			}
		}
		else
		{
			m_LeftPane->SetItemImage( p_Id , 1 , wxTreeItemIcon_Expanded );
			m_LeftPane->SetItemImage( p_Id , 1 , wxTreeItemIcon_SelectedExpanded );
		}
	}

	ShowItemAttributes( p_Id );	
}


int LDAPBrowser_MainFrame::OnSave( wxMenuEvent &p_Evt )
{
	
	int iRet = m_RightPane->OnSave();
	return iRet;
}


void LDAPBrowser_MainFrame::OnNewEntry( wxTreeItemId &p_Id )
{
	wxTreeItemId	Id;
	wxString		Dn;
	wxString		RDn;
	int				iRet;
	unsigned long	i;


	LDAPBrowser_CreateObjectDialog	*Dlg;
	
	Dlg = new LDAPBrowser_CreateObjectDialog( this );
	if( Dlg == NULL )
	{
		return;
	}

	RDn = GetDN( p_Id );

	Dlg->SetRDN( RDn );

	Dlg->SetSchema( &m_Schema );
	

	iRet = Dlg->ShowModal();
	if( iRet == wxID_CANCEL )
		return ;

	Dn = Dlg->GetDN();


	Id = m_LeftPane->AppendItem( p_Id , Dn , 0 , 0 );
	m_LeftPane->SetItemImage( p_Id , 0 , wxTreeItemIcon_Normal );
	m_LeftPane->SetItemImage( p_Id , 0 , wxTreeItemIcon_Selected );
	m_LeftPane->SetItemImage( p_Id , 1 , wxTreeItemIcon_Expanded );
	m_LeftPane->SetItemImage( p_Id , 1 , wxTreeItemIcon_SelectedExpanded );
	m_LeftPane->EnsureVisible( Id );

	m_RightPane->EmptyPane();

	m_RightPane->IsNew( true );

	wxString		FullDN ;

	FullDN = Dn ;
	if( RDn.length() > 0 )
	{
		FullDN += wxT(",");
		FullDN += RDn;
	}

	m_RightPane->SetDN( FullDN );

	m_RightPane->AddRootItem( FullDN.c_str() );

	long	EqualPos = Dn.Find( '=' );

	wxString			Naming ;
	wxString			Value ;
	StringVector	Values ;

	if( EqualPos >= 0 )
	{
		Naming = Dn.Left( EqualPos );
		Value = Dn.Right( Dn.Len() - ( EqualPos + 1 ) ) ;
		Values.push_back( Value ) ;

		m_RightPane->UIAddAttribute( Naming, Values, true );
	}
	for( i = 0 ; i < Dlg->m_MandatoriesList.size() ; i ++ )
	{
		if( Dlg->m_MandatoriesList[i].CmpNoCase( Naming ) != 0 
			|| Naming.length() == 0 )
		{
			Values.clear();
			m_RightPane->UIAddAttribute( Dlg->m_MandatoriesList[ i ], Values, true );
		}
	}
	for( i = 0 ; i < Dlg->m_OptionalsList.size() ; i ++ )
	{
		if( Dlg->m_OptionalsList[i].CmpNoCase( Naming ) != 0 
			|| Naming.length() == 0 )
		{
			Values.clear();
			m_RightPane->UIAddAttribute( Dlg->m_OptionalsList[i], Values, true );
			
		}
	}

	if( Dlg->m_ObjectClassStr.length() > 0 )
	{
		m_RightPane->UIAddAttributeValue( wxT("objectclass"), Dlg->m_ObjectClassStr, true );
	}
	m_RightPane->Expand();

	Dlg->Destroy();

}

int LDAPBrowser_MainFrame::OnDeleteEntry(wxTreeItemId &p_Id)
{
	wxString		Dn;
	wxString		Txt;

	m_RightPane->EmptyPane();
	
	Dn = GetDN( p_Id );

	int iRet = m_LDAP.Delete( Dn );

	return iRet;
}

//DEL int LDAPBrowser_MainFrame::OnModifyEntry(wxTreeItemId &p_Id , const wxString & p_NewDN )
//DEL {
//DEL 	wxString		Dn;
//DEL 
//DEL 	m_RightPane->EmptyPane();
//DEL 
//DEL 	Dn = GetDN( p_Id );
//DEL 	int iRet = m_LDAP.Rename( Dn, p_NewDN, false );
//DEL 
//DEL 	return iRet;
//DEL }

//DEL int LDAPBrowser_MainFrame::OnMoveEntry(wxTreeItemId &p_Id , const wxString & p_NewDN )
//DEL {
//DEL 	wxString		Dn;
//DEL 
//DEL 	m_RightPane->EmptyPane();
//DEL 
//DEL 	Dn = GetDN( p_Id );
//DEL 	int iRet = m_LDAP.Rename( Dn, p_NewDN, true );
//DEL 
//DEL 	return iRet;
//DEL }

void LDAPBrowser_MainFrame::OnItemSelected( wxTreeItemId &p_Id )
{
	int iRet ;
	/*
	* First check if the right pane's entry was being edited and not already saved
	*/
	if( m_RightPane->IsModified() )
	{
		iRet = ::wxMessageBox( _( "Some changes have not been saved\n( this is new entry or an empty attribute has been added ).\nDo you wish to continue ?" ),
			m_RightPane->GetDN().c_str(), wxYES_NO | wxNO_DEFAULT | wxICON_EXCLAMATION );

		if( iRet == wxYES )
		{
			/*
			* User does not want to save changes 
			*/
			m_RightPane->IsModified( false, true );
			m_RightPane->IsNew( false );
		};
		return ;
	}

	ShowItemAttributes( p_Id );
}

void LDAPBrowser_MainFrame::OnSearch(wxTreeItemId & p_Id, const wxString & p_SearchString)
{
	wxString		Dn ;
	
	Dn = GetDN( p_Id );

	
	int iRet = m_LDAP.Search( Dn , p_SearchString , 1 , LDAP_SCOPE_SUBTREE );
	if( iRet != LDAP_SUCCESS || m_LDAP.GetSize() <= 0 )
	{
		m_LeftPane->m_IsSearchResult = 0 ;
		
		wxMessageDialog Dlg( this , STR_MSG_SEARCH_NO_RESULT , STR_MSG_SEARCH_TITLE , wxOK );
		Dlg.ShowModal();
		return ;
	}

	m_LeftPane->m_IsSearchResult = 1 ;
	m_LeftPane->m_Base = Dn ;
	m_LeftPane->m_Request = p_SearchString ;

	m_LeftPane->DeleteAllItems();

	wxTreeItemId Id = m_LeftPane->AddRoot( STR_MSG_SEARCH_ROOT_ENTRY , 1 , 1 );

	m_LeftPane->SetItemImage( Id , 0 , wxTreeItemIcon_Normal );

	for( unsigned long j = 0 ; j < m_LDAP.GetSize() ; j ++ )
	{

		m_LeftPane->AppendItem( Id, m_LDAP.GetDN( j ).c_str(), 0, 0 );
		m_LeftPane->SetItemImage( Id, 0, wxTreeItemIcon_Normal );
		m_LeftPane->SetItemImage( Id, 0, wxTreeItemIcon_Selected );
		m_LeftPane->SetItemImage( Id, 1, wxTreeItemIcon_Expanded );
		m_LeftPane->SetItemImage( Id, 1, wxTreeItemIcon_SelectedExpanded );
	}
}

wxString LDAPBrowser_MainFrame::GetDN( wxTreeItemId & p_Id )
{
	wxTreeItemId	Id = p_Id ;
	wxString		Dn;

	if( m_LeftPane->m_IsSearchResult == 0 )
	{
		while ( Id != m_LeftPane->GetRootItem() && Id.IsOk())
		{
			Dn = Dn + m_LeftPane->GetItemText( Id );
			Id = m_LeftPane->GetParent( Id );
			Dn = Dn + wxT(",");
		}
		Dn += m_LeftPane->GetItemText( Id );
	}
	else
	{
		Dn = m_LeftPane->GetItemText( p_Id );
		
	}
	return Dn ;
}

void LDAPBrowser_MainFrame::OnSearchClick(wxMenuEvent &p_Evt)
{
	m_LeftPane->OnSearch( p_Evt );
}

void LDAPBrowser_MainFrame::OnExport(wxMenuEvent &p_Event)
{
	char			*AttributesToExport[] = { "*", NULL } ;
	wxFileDialog	FileDlg( 	this , _( "Choose an export file" ) , 
								wxT("") , wxT("Export.ldif") , 
								wxT("*.*") , wxSAVE );

	wxTreeItemId	Id;

	if( !m_LeftPane->m_IsSearchResult )
	{
		Id = m_LeftPane->GetSelection();
		if( !Id.IsOk() )
		{
			wxMessageDialog Dlg( this , _( "No Entry Selected" ) , _( "Export" ) );
			Dlg.ShowModal();
			return;
		}
	}

	if( FileDlg.ShowModal() == wxID_CANCEL ) 
	{
		return;
	}
	wxString	FileName = FileDlg.GetPath();
	LDIF_File	File ;
	int			iRet ;

	iRet = File.Open( m_LDAP.GetLDAPHandle(), m_LDAP.GetSrvType() );
	iRet = File.CreateFile( FileName.c_str() ) ;
	if( iRet != LDIF_NO_ERROR )
	{
		::wxMessageBox ( _( "Error creating file" ) , _( "Export" ), 
			wxOK | wxCENTER | wxICON_ERROR );
		return ;
	}
	if( m_LeftPane->m_IsSearchResult )
	{
		
		iRet = File.Search( m_LeftPane->m_Base, m_LeftPane->m_Request, 1, 
			LDAP_SCOPE_SUBTREE, AttributesToExport );
		if( iRet != LDAP_SUCCESS )
		{
			iRet = File.Search( m_LeftPane->m_Base, m_LeftPane->m_Request, 1, 
				LDAP_SCOPE_BASE, AttributesToExport );
		}
	}
	else
	{
		wxString Base;
		Base = GetDN( Id );
		
		iRet = File.Search( Base, wxT("objectClass=*"), 1, LDAP_SCOPE_SUBTREE, 
			AttributesToExport );
		if( iRet != LDAP_SUCCESS )
		{
			iRet = File.Search ( Base, wxT("objectClass=*"), 1, LDAP_SCOPE_BASE, 
				AttributesToExport );
		}
	}

	if( iRet != LDAP_SUCCESS )
	{
		wxString ErrMsg ( ldap_err2string( iRet ), *wxConvCurrent );
		::wxMessageBox( ErrMsg, _( "Export" ), wxOK | wxCENTER | wxICON_ERROR );
		return;
	}

	File.Export();
	File.Close();

}

void LDAPBrowser_MainFrame::OnNew( wxMenuEvent &p_Evt )
{
	wxGetApp().OnNew( p_Evt ) ;
}

void LDAPBrowser_MainFrame::SetSizeLimit( int SizeLimit )
{
	m_LDAP.SetSizeLimit( SizeLimit );
}

void LDAPBrowser_MainFrame::OnAbout( wxMenuEvent &p_Event )
{
	m_AboutDialog->Show(TRUE);
}

void LDAPBrowser_MainFrame::ShowItemAttributes( wxTreeItemId & p_Id )
{
	int			iRet ;
	wxString	Dn;

	m_RightPane->EmptyPane();

	/*
	* Search the property of the current selected item
	*/

	Dn = GetDN( p_Id );		

	iRet = m_LDAP.Search( Dn, wxT("objectClass=*"), 1, LDAP_SCOPE_BASE );
	if( iRet == 0 && m_LDAP.GetSize() > 0 )
	{
		m_RightPane->AddRootItem( m_LDAP.GetDN( 0 ).c_str() );

		m_RightPane->SetDN( m_LDAP.GetDN( 0 ).c_str() );

		for( long i = 0 ; i < m_LDAP.GetAttributeCount( 0 ) ; i ++ )
		{
			wxString Name;
			StringVector Values;
			m_LDAP.GetAttribute( 0, i, Name, Values );
			m_RightPane->UIAddAttribute( Name, Values, false );

			m_RightPane->Expand();
		}
	}

}

LDAP_Schema * LDAPBrowser_MainFrame::GetSchema()
{
	return &m_Schema ;
}

void LDAPBrowser_MainFrame::OnImport( wxMenuEvent &p_Event )
{
	wxFileDialog	FileDlg( this , _( "Choose an import file" ) , wxT("") , wxT("Import.ldif") , wxT("*.*") , wxOPEN );

	if( FileDlg.ShowModal() == wxID_CANCEL ) 
	{
		return;
	}

	wxString	FileName = FileDlg.GetPath();
	LDIF_File	File ;
	int			iRet ;

	iRet = File.OpenFile( FileName.c_str() ) ;
	if( iRet != LDIF_NO_ERROR )
	{
		::wxMessageBox ( _( "Error opening file" ) , _( "Import" ), wxOK | wxCENTER | wxICON_ERROR );
		return ;
	}
	File.Open( m_LDAP.GetLDAPHandle(), m_LDAP.GetSrvType() );
	File.Import();
	File.Close();
}

void LDAPBrowser_MainFrame::OnFileAddEntry( wxMenuEvent &p_Evt )
{
	m_LeftPane->OnPopupAdd( p_Evt );
}

void LDAPBrowser_MainFrame::OnFileDelEntry( wxMenuEvent &p_Evt )
{
	m_LeftPane->OnPopupDelete( p_Evt );
}

void LDAPBrowser_MainFrame::OnFileModDNEntry( wxMenuEvent &p_Evt )
{
	m_LeftPane->OnPopupModify( p_Evt );
}

void LDAPBrowser_MainFrame::OnFileMovDNEntry( wxMenuEvent &p_Evt )
{
	m_LeftPane->OnPopupMove( p_Evt );
}
void LDAPBrowser_MainFrame::OnFileAddAttribute( wxMenuEvent &p_Evt )
{
	m_RightPane->OnPopupAdd( p_Evt );
}

void LDAPBrowser_MainFrame::OnFileDelAttribute( wxMenuEvent &p_Evt )
{
	m_RightPane->OnPopupDelete( p_Evt );
}

void LDAPBrowser_MainFrame::OnFileModAttribute( wxMenuEvent &p_Evt )
{
	m_RightPane->OnPopupModify( p_Evt );
}

void LDAPBrowser_MainFrame::OnFileAddBinAttribute( wxMenuEvent &p_Evt )
{
	m_RightPane->OnPopupAddBin( p_Evt );
}
void LDAPBrowser_MainFrame::OnFileDelBinAttribute( wxMenuEvent &p_Evt )
{
	m_RightPane->OnPopupDelBin( p_Evt );
}
void LDAPBrowser_MainFrame::OnFileViewBinAttribute( wxMenuEvent &p_Evt )
{
	m_RightPane->OnPopupViewBin( p_Evt );
}
void LDAPBrowser_MainFrame::OnFileExportBinAttribute( wxMenuEvent &p_Evt )
{
	m_RightPane->OnPopupExportBin( p_Evt );
}
void LDAPBrowser_MainFrame::OnEditPreferences( wxMenuEvent &p_Evt )
{
	LDAPBrowser_PreferencesDialog *Dlg;
	int iRet;
	
	Dlg = new LDAPBrowser_PreferencesDialog( this );
	iRet = Dlg->ShowModal();

	Dlg->Destroy();
}
