/*! 
 *********************************************************************
 *  @file   : LDAPBrowser_OpenDialog.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 19/11/2002
 *
 *  Purpose : Implementation of methods for class LDAPBrowser_OpenDialog
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <wx/dialog.h>
#include <wx/config.h>
#include <wx/choice.h>
#include <wx/textctrl.h>
#include <wx/intl.h>
#include <wx/textdlg.h>
#include <wx/xrc/xmlres.h>

#include "../include/LDAPBrowser_Defines.h"
#include "../include/LDAPBrowser_OpenDialog.h"


BEGIN_EVENT_TABLE(LDAPBrowser_OpenDialog, wxDialog)
	EVT_BUTTON( XRCID( "button_connect_dialog_cancel" ), LDAPBrowser_OpenDialog::OnCancel )
	EVT_BUTTON( XRCID( "button_connect_dialog_ok" ), LDAPBrowser_OpenDialog::OnOk )
	EVT_BUTTON( XRCID( "button_connect_dialog_new" ), LDAPBrowser_OpenDialog::OnNewConfig )
	EVT_BUTTON( XRCID( "button_connect_dialog_delete" ), LDAPBrowser_OpenDialog::OnDeleteConfig )
	EVT_CHOICE( XRCID( "connect_dialog_configurations" ), LDAPBrowser_OpenDialog::OnSelect ) 
END_EVENT_TABLE()

LDAPBrowser_OpenDialog::LDAPBrowser_OpenDialog( wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos , const wxSize& size , long style , const wxString& name )
{
	wxXmlResource::Get()->LoadDialog( this, parent, wxT("connect_dialog") );

	m_Server		= ( wxTextCtrl * )FindWindowByName( wxT("connect_dialog_server"), this );
	m_Port			= ( wxTextCtrl * )FindWindowByName( wxT("connect_dialog_port"), this );
	m_RootDN		= ( wxTextCtrl * )FindWindowByName( wxT("connect_dialog_rootdn"), this );
	m_User			= ( wxTextCtrl * )FindWindowByName( wxT("connect_dialog_username"), this );
	m_Password		= ( wxTextCtrl * )FindWindowByName( wxT("connect_dialog_password"), this );
	m_Configuration = ( wxChoice * )FindWindowByName( wxT("connect_dialog_configurations"), this );
	m_ServerType    = ( wxChoice * )FindWindowByName( wxT("connect_dialog_servertype"), this );

	SetReturnCode( wxID_CANCEL );
	//wxConfigBase	*Config = wxConfigBase::Get( );
	wxConfig	*Config = new wxConfig( wxT(PACKAGE) );
	
	bool		bRet ;
	long		Index ;
	wxString	GroupName ;
	wxString	EntryName ;

	bRet = Config->GetFirstGroup( GroupName , Index ) ;
	while( bRet )
	{
		m_Configuration->Append( GroupName );
		bRet = Config->GetNextGroup( GroupName, Index ) ;
	}	
	delete Config ;

	for( long j = 0; LDAP_Server_TypeNames[ j ] != NULL; j ++ )
	{
		m_ServerType->Append( wxString( LDAP_Server_TypeNames[ j ], *wxConvCurrent ) );
	}
}

LDAPBrowser_OpenDialog::~LDAPBrowser_OpenDialog()
{


}

void LDAPBrowser_OpenDialog::OnOk( wxCommandEvent &event )
{

//	wxConfigBase			*Config = wxConfigBase::Get( );
	wxConfig				*Config = new wxConfig( wxT(PACKAGE) );
	wxTextEntryDialog		*ConfigName = NULL ;
	int						Selected ;

	TransferDataFromWindow();
	Selected = 	m_Configuration->GetSelection();
	if( Selected == -1 )
	{
		ConfigName = new wxTextEntryDialog( this , _( "Type a name for this configuration" ) );
		if( ConfigName->ShowModal() == wxID_OK && ConfigName->GetValue().length() > 0 )
		{
			m_Configuration->Append( ConfigName->GetValue() );
			m_Configuration->SetSelection( m_Configuration->Number() - 1 );
			Selected = 	m_Configuration->GetSelection();
		}
		else
		{
			delete Config;
			return;
		}
	}

	m_StrConfig = m_Configuration->GetString( Selected );

	m_StrServer = m_Server->GetValue();
	m_StrPort = m_Port->GetValue();
	m_StrBaseDN = m_RootDN->GetValue();
	m_StrUser = m_User->GetValue();
	m_StrPassword = m_Password->GetValue();

	m_SrvType = GetType( m_ServerType->GetString( m_ServerType->GetSelection() ) );

	Config->Write( m_StrConfig + wxT("/Server") , m_StrServer );
	Config->Write( m_StrConfig + wxT("/Port") , m_StrPort );
	Config->Write( m_StrConfig + wxT("/BaseDN") , m_StrBaseDN );
	Config->Write( m_StrConfig + wxT("/User") , m_StrUser );
	Config->Write( m_StrConfig + wxT("/Password") , m_StrPassword );
	Config->Write( m_StrConfig + wxT("/Type") , m_SrvType );
	Config->Flush( TRUE );

	delete Config;
	EndModal( wxID_OK );
}

void LDAPBrowser_OpenDialog::OnCancel( wxCommandEvent &WXUNUSED( event ) )
{
	EndModal( wxID_CANCEL );
}


void LDAPBrowser_OpenDialog::OnSelect(wxEvent &p_Evt)
{
	long Id ; 
	long Tmp;
	
	Id = p_Evt.GetId();
	Id = m_Configuration->GetSelection();
	if( Id != -1 )
	{
		//wxConfigBase *Config = wxConfigBase::Get( );	
		wxConfig *Config = new wxConfig( wxT(PACKAGE) );
		m_StrConfig = m_Configuration->GetString( Id );
		Config->Read( m_StrConfig + wxT("/Server"), &m_StrServer );
		Config->Read( m_StrConfig + wxT("/Port"), &m_StrPort );
		Config->Read( m_StrConfig + wxT("/BaseDN"), &m_StrBaseDN );
		Config->Read( m_StrConfig + wxT("/User"), &m_StrUser );
		Config->Read( m_StrConfig + wxT("/Password"), &m_StrPassword );
		Config->Read( m_StrConfig + wxT("/Type"), &Tmp );

		m_SrvType = ( LDAP_SERVER_TYPE )Tmp ;
		UpdateUI();
		delete Config;
	}
}

void LDAPBrowser_OpenDialog::OnNewConfig(wxCommandEvent &event)
{
	wxTextEntryDialog		*ConfigName = NULL ;

	ConfigName = new wxTextEntryDialog( this , _( "Type a name for this configuration" ) );
	if( ConfigName->ShowModal() == wxID_OK && ConfigName->GetValue().length() > 0 )
	{
		m_Configuration->Append( ConfigName->GetValue() );
		m_Configuration->SetSelection( m_Configuration->Number() - 1 );
	}
}

void LDAPBrowser_OpenDialog::OnDeleteConfig(wxCommandEvent &event)
{
	//wxConfigBase				*Config = wxConfigBase::Get( );
	wxConfig					*Config = new wxConfig( wxT(PACKAGE) );
	int							Selected ;
	unsigned long				i , nb ;
	StringVector	PreviousList;

	Selected = m_Configuration->GetSelection();
	if( Selected != -1 )
	{
		m_StrConfig = m_Configuration->GetString( Selected );
		nb = m_Configuration->GetCount();
		for( i = 0 ; i < nb ; i++ )
		{
			if( m_Configuration->GetString( i ) != m_StrConfig )
			PreviousList.insert( PreviousList.end() , m_Configuration->GetString( i ).c_str() ) ;
		}
		
		m_Configuration->Clear();
		for( i = 0 ; i < PreviousList.size() ; i ++ )
		{
			m_Configuration->Append( PreviousList[i].c_str() );
		}
		Config->DeleteGroup(  m_StrConfig );
		Config->Flush();
	}

	delete Config ;
}

void LDAPBrowser_OpenDialog::UpdateUI()
{
	m_Server->SetValue( m_StrServer );
	m_Port->SetValue( m_StrPort );
	m_User->SetValue( m_StrUser );
	m_Password->SetValue( m_StrPassword );
	m_RootDN->SetValue( m_StrBaseDN );
	m_ServerType->SetSelection( GetTypeNameIndex( m_SrvType ) );
}

LDAP_SERVER_TYPE LDAPBrowser_OpenDialog::GetType(const wxString & p_ServerType)
{
	LDAP_SERVER_TYPE Ret = LDAP_Server_Types[ 0 ] ;
	for( long i = 0; LDAP_Server_TypeNames[ i ] != NULL; i++ )
	{
		if( p_ServerType.CmpNoCase( wxString( LDAP_Server_TypeNames[ i ], *wxConvCurrent ) ) == 0 )
		{
			Ret = LDAP_Server_Types[ i ] ;
			break;
		}
	}
	return Ret ;
}

int LDAPBrowser_OpenDialog::GetTypeNameIndex(LDAP_SERVER_TYPE p_Type)
{
	int Ret = -1 ;

	for( long i = 0; LDAP_Server_Types[ i ] != ( LDAP_SERVER_TYPE ) NULL ; i++ )
	{
		if( LDAP_Server_Types[ i ] == p_Type )
		{
			Ret = i ;
		}
	}
	return Ret;
}
