/*! 
 *********************************************************************
 *  @file   : LDAPBrowser_PreferencesDialog.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 19/11/2002
 *
 *  Purpose : Implementation of methods for class LDAPBrowser_PreferencesDialog
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <wx/intl.h>
#include <wx/choice.h>
#include <wx/textctrl.h>
#include <wx/config.h>
#include <wx/xrc/xmlres.h>

#include "../include/LDAPBrowser_Defines.h"
#include "../include/LDAPBrowser_PreferencesDialog.h"
#include "../include/LDAPBrowser_MainFrame.h"


BEGIN_EVENT_TABLE(LDAPBrowser_PreferencesDialog, wxDialog)
    EVT_BUTTON( XRCID( "button_preferences_dialog_ok" ), LDAPBrowser_PreferencesDialog::OnOk )
    EVT_BUTTON( XRCID( "button_preferences_dialog_cancel" ), LDAPBrowser_PreferencesDialog::OnCancel )
END_EVENT_TABLE()

LDAPBrowser_PreferencesDialog::LDAPBrowser_PreferencesDialog( wxWindow* parent )
{    
	m_LanguageStrings.push_back( _("(System default)") );
	m_LanguageStrings.push_back( _("French") );
/*	m_LanguageStrings[2] = _("German");
	m_LanguageStrings[3] = _("Russian");
	m_LanguageStrings[4] = _("English");
	m_LanguageStrings[5] = _("English (U.S.)");
	m_LanguageStrings[6] = _("Elsassich" );
*/
	// each entry must correspond to m_LanguageStrings[]
	m_LanguageCodes.push_back( wxLANGUAGE_DEFAULT );
	m_LanguageCodes.push_back( wxLANGUAGE_FRENCH );
/*	m_LanguageCodes[2] = wxLANGUAGE_GERMAN;
	m_LanguageCodes[3] = wxLANGUAGE_RUSSIAN;
	m_LanguageCodes[4] = wxLANGUAGE_ENGLISH;
	m_LanguageCodes[5] = wxLANGUAGE_ENGLISH_US;
	m_LanguageCodes[6] = wxLANGUAGE_ELSASSICH;
*/
	//wxConfigBase	*Conf = wxConfigBase::Get();
	wxConfig	*Conf = new wxConfig( wxT(PACKAGE) );

	if ( !(wxXmlResource::Get()->LoadDialog( this, parent, wxT("preferences_dialog") )) )
	{
		Create( parent, -1, wxT("Preferences") );
	}

	m_Language 			= ( wxChoice * )	FindWindowByName( wxT("preferences_language"), this );
	if ( !m_Language )
	{
		::wxLogError( _("Trouble inititializing m_Language\n") );
	}
	m_SearchSizeLimit 	= ( wxTextCtrl * )	FindWindowByName( wxT("preferences_search_size"), this );
	if ( !m_SearchSizeLimit )
	{
		::wxLogError( _("Trouble inititializing m_SearchSizeLimit\n") );
	}
	
	// Fill the controls with values
	if ( m_Language )
	{
		for ( unsigned int i = 0; i < m_LanguageStrings.size(); i++ )
		{
			m_Language->Append( m_LanguageStrings[i].c_str(), &m_LanguageCodes[i] );
		}
	}
	
	// We find and select the entry corresponding to what's in the config file
	int curLangIndex, curLangCode;
	curLangCode = Conf->Read( wxT("Lang"), wxLANGUAGE_DEFAULT );
	curLangIndex = GetLanguageIndexFromCode( curLangCode );
	if ( m_Language )
		m_Language->SetSelection( curLangIndex );
	
	int			SizeLimit ;
	wxString	StrSizeLimit;

	Conf->Read( wxT("SizeLimit"), &SizeLimit ) ;

	StrSizeLimit.Printf( wxT("%d"), SizeLimit );
	
	if ( m_SearchSizeLimit )
		m_SearchSizeLimit->SetValue( StrSizeLimit ); 

	delete Conf ;
}

LDAPBrowser_PreferencesDialog::~LDAPBrowser_PreferencesDialog()
{    
}

void LDAPBrowser_PreferencesDialog::OnOk( wxCommandEvent &p_Evt )
{
	long SizeLimit ;
	wxString StrSizeLimit ;
	//wxConfigBase			*Config = wxConfigBase::Get( );
	wxConfig			*Config = new wxConfig( wxT(PACKAGE) );

	StrSizeLimit = m_SearchSizeLimit->GetValue();
	if( StrSizeLimit.Len() != 0 )
	{
		StrSizeLimit.ToLong( &SizeLimit );
		LDAPBrowser_MainFrame * MainFrame = (LDAPBrowser_MainFrame *)GetParent();
		MainFrame->SetSizeLimit( SizeLimit );
		Config->Write( wxT("SizeLimit"), SizeLimit );
	}
	
	int lang = m_Language->GetSelection();
	long langCode = *(long *)m_Language->GetClientData( lang );
	Config->Write( wxT("Lang"), langCode );

	Config->Flush();

	delete Config ;
	EndModal( wxID_OK );
}

void LDAPBrowser_PreferencesDialog::OnCancel( wxCommandEvent &p_Evt )
{
		EndModal( wxID_CANCEL );
}

int LDAPBrowser_PreferencesDialog::GetLanguageIndexFromCode( int p_langCode )
{
	for ( unsigned int i = 0; i < m_LanguageCodes.size(); i++ )
	{
		if ( m_LanguageCodes[i] == p_langCode )
		{
			return i;
		}
	}
	return 0;
}
