/*! 
 *********************************************************************
 *  @file   : LDAPBrowser_RightPane.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 19/11/2002
 *
 *  Purpose : Implementation of methods for class LDAPBrowser_RightPane
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <algorithm>

#include <wx/frame.h>
#include <wx/menu.h>
#include <wx/listctrl.h>
#include <wx/treectrl.h>
#include <wx/intl.h>
#include <wx/msgdlg.h>
#include <wx/textdlg.h>
#include <wx/choicdlg.h>
#include <wx/filedlg.h>
#include <wx/ffile.h>
#include <wx/xrc/xmlres.h>

#include "../include/LDAPBrowser_Defines.h"
#include "../include/Utils.h"
#include "../include/LDAPBrowser_MainFrame.h"
#include "../include/LDAPBrowser_RightPane.h"
#include "../include/LDIF_File.h"
#include "../include/LB_BinaryDlg.h"


BEGIN_EVENT_TABLE( LDAPBrowser_RightPane , wxTreeCtrl )
EVT_TREE_BEGIN_LABEL_EDIT( -1,LDAPBrowser_RightPane::OnBeginEdit )    
EVT_TREE_END_LABEL_EDIT( -1 , LDAPBrowser_RightPane::OnEndEdit )    
EVT_RIGHT_DOWN( LDAPBrowser_RightPane::OnRightMouseClick )
EVT_MENU( XRCID( "right_popup_menu_mod" ), LDAPBrowser_RightPane::OnPopupModify )
EVT_MENU( XRCID( "right_popup_menu_add" ), LDAPBrowser_RightPane::OnPopupAdd )
EVT_MENU( XRCID( "right_popup_menu_addbinary" ), LDAPBrowser_RightPane::OnPopupAddBin )
EVT_MENU( XRCID( "right_popup_menu_viewbinary" ), LDAPBrowser_RightPane::OnPopupViewBin )
EVT_MENU( XRCID( "right_popup_menu_delbinary" ), LDAPBrowser_RightPane::OnPopupDelBin )
EVT_MENU( XRCID( "right_popup_menu_exportbinary" ), LDAPBrowser_RightPane::OnPopupExportBin )
EVT_MENU( XRCID( "right_popup_menu_del" ), LDAPBrowser_RightPane::OnPopupDelete )
EVT_MENU( XRCID( "right_popup_menu_save" ), LDAPBrowser_RightPane::OnPopupSave )
END_EVENT_TABLE()

LDAPBrowser_RightPane::LDAPBrowser_RightPane(wxWindow* parent, wxWindowID id, const wxPoint& pos , const wxSize& size , long style , const wxValidator& validator , const wxString& name ) 
: wxTreeCtrl(parent,id,pos,size,style,validator,name)
{
	m_IsNew = false;
}

LDAPBrowser_RightPane::~LDAPBrowser_RightPane()
{
	EmptyPane();
}

void LDAPBrowser_RightPane::OnBeginEdit( wxTreeEvent &p_Evt )
{
	if( GetSelection() == GetRootItem() )
	{
		p_Evt.Veto();
	}
}
void LDAPBrowser_RightPane::OnEndEdit( wxTreeEvent & p_Evt )
{
	wxString			Current;
	wxString			Previous;
	wxTreeItemId		ParentID ;
	wxTreeItemId		CurrentSelectedItem ;
	int					iRet;
	
	wxString			Name;
	StringVector		Values ;
	wxTreeItemId		Id ;
	long				Cookie;

	Previous = GetItemText( p_Evt.GetItem() );
	Current = p_Evt.GetLabel();
	CurrentSelectedItem = GetSelection();
	ParentID = GetParent( CurrentSelectedItem );

	if( Current != Previous && Current.length() > 0 )
	{
		if( ParentID == GetRootItem() )
		{
			/*
			* Current selected item is an attribute
			*/
			Name = Previous.c_str();
			iRet = Mods_AddAttribute( LDAP_MOD_DELETE, Name, Values );

			Id = GetFirstChild( CurrentSelectedItem, Cookie );
			while( Id.IsOk() && iRet == LDAP_SUCCESS )
			{
				Values.push_back( GetItemText( Id ).c_str() );
				Id = GetNextChild( CurrentSelectedItem, Cookie );
			}

			Name = Current.c_str();
			iRet = Mods_AddAttribute( LDAP_MOD_ADD, Name, Values );
			if( iRet == LDAP_SUCCESS )
			{
				iRet = OnSave();
			}
			if( iRet != LDAP_SUCCESS )
			{
				SetItemText( CurrentSelectedItem, Previous );
			}
		}
		else
		{
			/*
			* Current selected item is an attribute value
			*/
			Name = GetItemText( ParentID ).c_str();
			Values.push_back( Current.c_str() );
			iRet = Mods_AddAttribute( LDAP_MOD_ADD, Name, Values );
			if( iRet == LDAP_SUCCESS )
			{
				Values.clear();
				Values.push_back( Previous.c_str() );
				iRet = Mods_AddAttribute( LDAP_MOD_DELETE, Name, Values );	
				iRet = OnSave();
				if( iRet != LDAP_SUCCESS )
				{
					SetItemText( CurrentSelectedItem, Previous );
				}
			}
		}
	}
}

void LDAPBrowser_RightPane::OnRightMouseClick( wxMouseEvent & p_Evt )
{
	wxMenu	*Popup = NULL;
	wxPoint	Position;
	wxTreeItemId				ParentID ;
	wxTreeItemId				RootID ;
	wxTreeItemId				CurrentSelectedItem ;

	Position = p_Evt.GetPosition();
	CurrentSelectedItem = HitTest( Position );

	SelectItem( CurrentSelectedItem );
	if( CurrentSelectedItem.IsOk() )
	{
		Popup = wxXmlResource::Get()->LoadMenu( wxT("right_popup_menu") );

		RootID = GetRootItem() ;
		ParentID = GetParent( CurrentSelectedItem ) ;
		if ( CurrentSelectedItem == RootID )
		{
			// The selected item is the object itself
			Popup->Enable( XRCID( "right_popup_menu_add" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_mod" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_del" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_addbinary" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_delbinary" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_exportbinary" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_viewbinary" ), FALSE );
		}
		else if ( ParentID == RootID )
		{
			// The selected item is an attribute's name
			Popup->Enable( XRCID( "right_popup_menu_add" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_mod" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_del" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_addbinary" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_delbinary" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_exportbinary" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_viewbinary" ), FALSE );
		}
		else
		{
			// The selected item is an attribute's value.
			Popup->Enable( XRCID( "right_popup_menu_add" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_mod" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_del" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_addbinary" ), FALSE );
			Popup->Enable( XRCID( "right_popup_menu_delbinary" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_exportbinary" ), TRUE );
			Popup->Enable( XRCID( "right_popup_menu_viewbinary" ), TRUE );
		}
		if( IsModified() )
		{
			Popup->Enable( XRCID( "right_popup_menu_save" ), TRUE );
		}
		else
		{
			Popup->Enable( XRCID( "right_popup_menu_save" ), FALSE );
		}

		PopupMenu( Popup , Position ) ;
	}
	delete Popup;
}

void LDAPBrowser_RightPane::OnPopupModify(wxMenuEvent &p_Evt)
{
	EditLabel( GetSelection() );
}

void LDAPBrowser_RightPane::OnPopupAdd( wxMenuEvent &p_Evt )
{
	wxTreeItemId				ParentId ;
	wxTreeItemId				AttributeID ;
	wxTreeItemId				RootID ;
	wxTreeItemId				CurrentSelectedItem ;
	wxString					ItemValue ;
	int							iRet = LDAP_SUCCESS;
	wxTreeItemId				Id ;
	wxString					Name;
	StringVector				Values ;
	long						Cookie;
	bool						Found = false;

	CurrentSelectedItem = GetSelection() ;
	ParentId = GetParent( CurrentSelectedItem ) ;
	RootID = GetRootItem();
	if( CurrentSelectedItem == RootID )
	{
		/*
		* Add a new attribute
		*/
		if( UIGetAttributeName( ItemValue ) != 0 )
		{
			ItemValue = ::wxGetTextFromUser( _( "Enter the attribute to add" ), _( "Add Attribute" ) );
		}
		if( ItemValue.Length() > 0 )
		{
			/*
			* Check if the attribute already exists in the tree
			*/
			AttributeID = GetFirstChild( RootID, Cookie );
			while( !Found && AttributeID.IsOk() )
			{
				if( GetItemText( AttributeID ) == ItemValue )
				{
					Found = true;
				}
				AttributeID = GetNextChild( RootID, Cookie );
			}
			if( Found )
			{
				::wxMessageBox( _( "This attribute already exists" ), _( "Error" ), 
					wxOK | wxCENTER );
				return;
			}
			else
			{
			}
		}
	}
	else if( ParentId == RootID )
	{
		/*
		* Add a new attribute value
		*/
		ItemValue = ::wxGetTextFromUser( _( "Enter the value to add" ), _( "Add value" ) );
		if( ItemValue.Length() > 0 )
		{
			Name = GetItemText( CurrentSelectedItem ).c_str();
			Values.push_back( ItemValue );
			iRet = Mods_AddAttribute( LDAP_MOD_ADD, Name, Values );
			if( iRet == LDAP_SUCCESS && !m_IsNew )
			{
				iRet = OnSave();
			}
		}
	}
	else
	{
		::wxMessageBox( _( "Select an attribute or the root first" ), _( "Error" ), 
			wxOK | wxCENTER );
		return ;
	}

	if( ItemValue.Length() >  0 && iRet == LDAP_SUCCESS )
	{
		Id = AppendItem( CurrentSelectedItem, ItemValue );
		EnsureVisible( Id );
	}
}

void LDAPBrowser_RightPane::OnPopupDelete(wxMenuEvent &p_Evt)
{
	wxTreeItemId			ParentID ;
	wxTreeItemId			CurrentSelectedItem ;
	wxString				Name;
	StringVector			Values;

	int						iRet = LDAP_SUCCESS;
	wxString				ItemValue;
	
	CurrentSelectedItem = GetSelection();
	if( CurrentSelectedItem == GetRootItem() )
	{
		::wxMessageBox( _( "Root cannot be deleted" ), _( "Error" ), wxOK | wxCENTER );
		return ;
	}

	ParentID = GetParent( CurrentSelectedItem );

	if( ParentID == GetRootItem() )
	{
		/*
		* Current selected item is an attribute
		*/
		Name = GetItemText( CurrentSelectedItem ).c_str() ;
		if( GetChildrenCount( CurrentSelectedItem ) == 0 )
		{
			Delete( CurrentSelectedItem );
			return;
		}
		else
		{
			Values.clear();
			Name = GetItemText( CurrentSelectedItem ).c_str() ;
			iRet = Mods_AddAttribute( LDAP_MOD_DELETE, Name, Values );
		}
	}
	else
	{
		/*
		* Current selected item is an attribute value
		*/
		Name = GetItemText( ParentID ).c_str() ;
		ItemValue = GetItemText( CurrentSelectedItem ) ;
		Values.push_back( ItemValue.c_str() );
		iRet = Mods_AddAttribute( LDAP_MOD_DELETE, Name, Values );
	}
	
	if( iRet == LDAP_SUCCESS && !m_IsNew )
	{
		iRet = OnSave();
	}

	if( iRet == LDAP_SUCCESS )
	{
		Delete( CurrentSelectedItem );
	}
}

void LDAPBrowser_RightPane::OnPopupSave(wxMenuEvent &p_Evt)
{
	OnSave();
}

int LDAPBrowser_RightPane::OnSave()
{
	LDAPBrowser_MainFrame	*Parent ;

	int						iRet = 0;

	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();
	/*
	* just call the parent OnSave method
	*/
	if( IsModified() )
	{
		iRet = Parent->m_LDAP.WriteMods( m_DN, m_Mods, IsNew() );
		if( iRet == LDAP_SUCCESS )
		{
			IsNew( false );
			IsModified( false, false );
		}
		else
		{
			if( !IsNew() )
			{
				IsModified( false, false );
			}
			wxString ErrMsg ( ldap_err2string( iRet ), *wxConvCurrent );
			::wxMessageBox( ErrMsg, _( "LDAP Error" ), wxCENTER | wxICON_ERROR );
		}
	}
	return iRet;
}


bool LDAPBrowser_RightPane::IsModified()
{
	long			Cookie ;
	wxTreeItemId	AttributeID ;
	wxTreeItemId	RootID ;
	bool			Found = false;

	if( m_Mods.size() > 0 )
	{
		return true;
	}

	RootID = GetRootItem() ;
	AttributeID = GetFirstChild( RootID, Cookie );
	while( !Found && AttributeID.IsOk() )
	{
		if( GetChildrenCount( AttributeID ) == 0 )
		{
			Found = true;
		}
		AttributeID = GetNextChild( RootID, Cookie );
	}
	if( Found )
	{
		return true;
	}
	return false;
}

void LDAPBrowser_RightPane::IsModified( bool p_State, bool p_CheckEmptyAtt )
{
	long			Cookie;
	wxTreeItemId	AttributeID ;
	wxTreeItemId	RootID ;
	wxTreeItemId	IDToDelete ;
	bool			DeleteID = false;

	if( p_State == false )
	{
		EmptyMods();
	}

	if( !p_CheckEmptyAtt )
	{
		return ;
	}
	RootID = GetRootItem() ;
	AttributeID = GetFirstChild( RootID, Cookie );
	while( AttributeID.IsOk() )
	{
		if( GetChildrenCount( AttributeID ) == 0 )
		{
			IDToDelete = AttributeID ;
			DeleteID = true;
		}
		else
		{
			DeleteID = false;
		}
		
		AttributeID = GetNextChild( RootID, Cookie );
		if( DeleteID )
		{
			Delete( IDToDelete );
		}
	}
}

bool LDAPBrowser_RightPane::IsNew()
{
	return m_IsNew ;
}

void LDAPBrowser_RightPane::IsNew( bool p_IsNew )
{
	m_IsNew = p_IsNew ;
}

void LDAPBrowser_RightPane::EmptyPane( bool p_DeleteUI )
{
	if( p_DeleteUI )
	{
		DeleteAllItems();
		m_DN = wxT("");
	}

}

void LDAPBrowser_RightPane::SetDN(const wxString &p_DN)
{
	m_DN = p_DN ;
}

wxString LDAPBrowser_RightPane::GetDN()
{
	return m_DN ;
}

int LDAPBrowser_RightPane::UIGetAttributeName( wxString & p_AttributeName)
{
	LDAP_Schema					*Schema = NULL ;
	wxString					ClassName ;
	LDAPBrowser_MainFrame		*Parent = NULL ;
	StringVector			Attributes ;
	StringVector			AttributesTmp ;
	StringVector::iterator iter;
	wxString					EntryValue ;
	wxTreeItemId				CurrentID;
	wxTreeItemId				RootID ;
	wxTreeItemId				ObjectClassID;
	long						Cookie;
	bool						Found = false;

	/*
	* Try to find the "objectClass" attribute and then all its values
	*/

	RootID = GetRootItem() ;
	CurrentID = GetFirstChild( RootID, Cookie ) ;
	while( CurrentID.IsOk() && !Found )
	{
		EntryValue = GetItemText( CurrentID );
		if( EntryValue.CmpNoCase( wxT("objectclass") ) == 0 )
		{
			ObjectClassID = CurrentID ;
			Found = true ;
		}
		else
		{
			CurrentID = GetNextChild( RootID, Cookie );
		}
	}

	if( !Found )
	{
		return -1 ;
	}


	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();
	Schema = Parent->GetSchema() ;

	if( !Schema->IsOpen() )
	{
		return -1;
	}

	CurrentID = GetFirstChild( ObjectClassID, Cookie );
	while( CurrentID.IsOk() )
	{
		AttributesTmp = Schema->GetObjectClassAttributes( GetItemText( CurrentID ).c_str(),
			ATTRIBUTE_TYPE_ALL ) ;
		for( iter =AttributesTmp.begin(); iter != AttributesTmp.end(); iter ++ )
		{
			if( std::find( Attributes.begin(), Attributes.end(), ( *iter ) ) == Attributes.end() )
			{
				Attributes.push_back( ( *iter ) );
			}
		}
		CurrentID = GetNextChild( ObjectClassID, Cookie );
	}
	
	if( Attributes.size() <= 0 )
	{
		return -1 ;
	}

	std::sort( Attributes.begin(), Attributes.end() ) ;
	std::unique( Attributes.begin(), Attributes.end() ) ;

	wxArrayString							Choices ;

	for( iter = Attributes.begin(); iter != Attributes.end() ; iter ++ )
	{
		Choices.Add( ( *iter ).c_str() );
	}
	p_AttributeName = ::wxGetSingleChoice( _( "Select the attribute to add..." ), 
		_( "Add attribute" ), Choices );
	
	return 0;
}

void LDAPBrowser_RightPane::OnPopupAddBin( wxMenuEvent &p_Evt )
{
	wxTreeItemId				ParentId ;
	wxTreeItemId				Id ;
	wxTreeItemId				CurrentSelectedItem ;
	int							iRet;
	wxString					FileName ;
	
	CurrentSelectedItem = GetSelection() ;
	ParentId = GetParent( CurrentSelectedItem ) ;

	if( CurrentSelectedItem == GetRootItem() )
	{
		/*
		* Only on a new attribute's value
		*/
		return;
	}
	else if( ParentId != GetRootItem() )
	{
		::wxMessageBox( _( "Select an attribute first" ), _( "Error" ), wxOK | wxCENTER );
		return ;
	}

	FileName = ::wxFileSelector( _( "Select the file to import as a binary value" ), 
		wxT(""), wxT(""), wxT(""), wxT("*.*"), wxOPEN | wxFILE_MUST_EXIST );

	if( FileName.Length() == 0 )
	{
		return;
	}

	wxFFile	File( FileName.c_str(), "r+b" );
	
	if( !File.IsOpened() )
	{
		::wxMessageBox( _( "Could not open file" ), _("Binary Add"), 
			wxOK | wxCENTER | wxICON_EXCLAMATION );
		return ;
	}

	berval	*Buffer = NULL ;
	size_t	FileSize;
	size_t	lRet;

	FileSize = File.Length();

	if( FileSize > 0 )
	{
		Buffer = new berval ;
		if( Buffer == NULL )
		{
			::wxMessageBox( _( "Not enough memory" ), _( "Error" ), 
				wxOK | wxCENTER | wxICON_EXCLAMATION ) ;
			return ;
		}
		Buffer->bv_val = new char[ FileSize ];
		if( Buffer->bv_val == NULL )
		{
			delete Buffer ;
			::wxMessageBox( _( "Could not allocate memory" ), wxT("Binary Add"), 
				wxOK | wxCENTER | wxICON_EXCLAMATION );
			return ;
		}

		Buffer->bv_len = FileSize ;
		lRet = File.Read( Buffer->bv_val, Buffer->bv_len );
		if( lRet < FileSize )
		{
			delete []Buffer->bv_val ;
			delete Buffer ;
			::wxMessageBox( _( "Could not read data from file" ), _( "Error" ), 
				wxOK | wxCENTER | wxICON_EXCLAMATION );
			return;
		}
		wxString Printable;
		ToBinaryPrint( Buffer->bv_val, Buffer->bv_len, Printable );

		iRet = BinaryValueOp( LDAP_MOD_ADD, GetItemText( CurrentSelectedItem ), Buffer );
		if( iRet == LDAP_SUCCESS )
		{
			Buffer = NULL ;
			iRet = OnSave();
			if( iRet == LDAP_SUCCESS )
			{
				Id = AppendItem( CurrentSelectedItem, Printable.c_str() );
				EnsureVisible( Id );
			}
		}
		else
		{
			delete []Buffer->bv_val ;
			delete Buffer ;
		}
	}
	else
	{
		::wxMessageBox( _( "File is zero sized" ), _( "Error" ), 
			wxOK | wxCENTER | wxICON_EXCLAMATION );
	}
	return ;
}

void LDAPBrowser_RightPane::OnPopupExportBin( wxMenuEvent &p_Evt )
{
	wxTreeItemId				ParentID ;
	wxTreeItemId				RootID ;
	wxTreeItemId				CurrentSelectedItem ;
	wxString					FileName ;
	LDAPBrowser_MainFrame		*Parent ;
	long						Pos = -1 ;
	
	CurrentSelectedItem = GetSelection() ;
	RootID = GetRootItem() ;
	ParentID = GetParent( CurrentSelectedItem ) ;

	if( ParentID == RootID || CurrentSelectedItem == RootID )
	{
		/*
		 * The user must click on a item's attribute
		 */
		return;
	}

	FileName = ::wxFileSelector( _( "Export the binary value as" ), wxT(""), 
		wxT(""), wxT(""), wxT("*.*"), wxSAVE );

	if( FileName.Length() == 0 )
	{
		return;
	}

	wxFFile	File( FileName.c_str(), "w+b" );
	
	if( !File.IsOpened() )
	{
		::wxMessageBox( _( "Could not create file" ), _("Binary Export"), 
			wxOK | wxCENTER | wxICON_EXCLAMATION );
		return ;
	}

	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();

	Pos = GetAttributeIndex( CurrentSelectedItem );
	if( Pos == -1 )
	{
		return ;
	}

	berval		*BinValue = NULL ;
	wxString	AttributeName = GetItemText( ParentID );

	BinValue = Parent->m_LDAP.GetAttributeBinValue( AttributeName.c_str(), Pos );
	if( BinValue != NULL )
	{
		File.Write( BinValue->bv_val,BinValue->bv_len );
		if( BinValue != NULL && BinValue->bv_val != NULL )
		{
			delete []BinValue->bv_val ;
			delete BinValue;
		}
	}
	File.Flush();
	File.Close();
/*
	berval	*Buffer = NULL ;
	size_t	FileSize;
	size_t	lRet;

	FileSize = File.Length();

	if( FileSize > 0 )
	{
		Buffer = new berval ;
		if( Buffer == NULL )
		{
			::wxMessageBox( _( "Not enough memory" ), _( "Error" ), 
				wxOK | wxCENTER | wxICON_EXCLAMATION ) ;
			return ;
		}
		Buffer->bv_val = new char[ FileSize ];
		if( Buffer->bv_val == NULL )
		{
			delete Buffer ;
			::wxMessageBox( _( "Could not allocate memory" ), "Binary Add", 
				wxOK | wxCENTER | wxICON_EXCLAMATION );
			return ;
		}

		Buffer->bv_len = FileSize ;
		lRet = File.Read( Buffer->bv_val, Buffer->bv_len );
		if( lRet < FileSize )
		{
			delete []Buffer->bv_val ;
			delete Buffer ;
			::wxMessageBox( _( "Could not read data from file" ), _( "Error" ), 
				wxOK | wxCENTER | wxICON_EXCLAMATION );
			return;
		}
		wxString Printable;
		ToBinaryPrint( Buffer->bv_val, Buffer->bv_len, Printable );

		iRet = BinaryValueOp( LDAP_MOD_ADD, GetItemText( m_CurrentSelectedItem ), Buffer );
		if( iRet == LDAP_SUCCESS )
		{
			Buffer = NULL ;
			iRet = OnSave();
			if( iRet == LDAP_SUCCESS )
			{
				Id = AppendItem( m_CurrentSelectedItem, Printable.c_str() );
				EnsureVisible( Id );
			}
		}
		else
		{
			delete []Buffer->bv_val ;
			delete Buffer ;
		}
	}
	else
	{
		::wxMessageBox( _( "File is zero sized" ), _( "Error" ), 
			wxOK | wxCENTER | wxICON_EXCLAMATION );
	}
*/
	return ;
}

int LDAPBrowser_RightPane::BinaryValueOp( long p_Op, const wxString &p_Attribute, berval *p_Value )
{
	LDAPMod	*Mod = NULL ;

	Mod = new LDAPMod ;
	if( Mod == NULL )
	{
		return -1;
	}

	Mod->mod_op = p_Op | LDAP_MOD_BVALUES;
	Mod->mod_type = NULL ;
	Mod->mod_type = new char[ p_Attribute.Length() + 1 ];
	if( Mod->mod_type == NULL )
	{
		delete Mod ;
		return -1;
	}
	strcpy( Mod->mod_type, p_Attribute.mb_str() );

	Mod->mod_bvalues = NULL ;
	Mod->mod_values = NULL ;

	Mod->mod_bvalues = new berval *[ 2 ];
	if( Mod->mod_bvalues == NULL )
	{
		delete []Mod->mod_type ;
		delete Mod ;
		return -1;
	}

	Mod->mod_bvalues[ 0 ] = p_Value ;
	Mod->mod_bvalues[ 1 ] = NULL ;

	m_Mods.insert( m_Mods.end(), Mod );
	return 0;

}

int LDAPBrowser_RightPane::UIAddAttribute(const wxString &p_Name, StringVector &p_Values, bool p_IsNew)
{
	wxTreeItemId	RootID;
	wxTreeItemId	AttributeID;
	int				iRet = LDAP_SUCCESS;

	RootID = GetRootItem();
	AttributeID = AppendItem( RootID, p_Name.c_str() );

	if( p_IsNew && p_Values.size() > 0 )
	{
		iRet = Mods_AddAttribute( LDAP_MOD_ADD, p_Name, p_Values );
	}
	
	if( iRet == LDAP_SUCCESS )
	{
 		for( unsigned long i = 0; i < p_Values.size(); i++ )
 		{
			AppendItem( AttributeID, p_Values[ i ].c_str() );
 		}
	}
	return iRet;
}

int LDAPBrowser_RightPane::Mods_AddAttribute( int p_Op, const wxString &p_Name, StringVector &p_Values )
{
	LDAPMod						*Mod = NULL ;
	LDAPMod						*Previous = NULL ;
	char						**PreviousVals = NULL;
	long						NbVal ;
	unsigned long				i;
	CSTL_ModsVector::iterator	iter;
	

	if( p_Name.length() == 0 )
	{
		return 0;
	}
	for( iter = m_Mods.begin(); iter != m_Mods.end(); iter ++ )
	{
		wxString CurMod( ( *iter )->mod_type, *wxConvCurrent);
 		if( ( *iter )->mod_op == p_Op && 
 			p_Name.CmpNoCase( CurMod ) == 0 )
 		{
 			Previous = ( *iter ) ;
 			break;
 		}
 	}

	/*
	* Search for a previous operation with the same attribute
	*/
	if( Previous != NULL )
	{
		Mod = Previous ;
	}
	else
	{
		Mod = new LDAPMod ;
		if( Mod == NULL )
		{
			return -1;
		}

		Mod->mod_op = p_Op ;
		Mod->mod_type = NULL ;
		Mod->mod_type = new char[ p_Name.length() + 1 ];
		if( Mod->mod_type == NULL )
		{
			delete Mod ;
			return -1;
		}
		strcpy( Mod->mod_type, p_Name.mb_str() );
	}
	NbVal = 0;

	if( Previous == NULL )
	{
		Mod->mod_values = NULL ;
	}
	else if( Previous->mod_values != NULL )
	{
		for( NbVal = 0; Previous->mod_values[ NbVal ] != NULL; NbVal ++)
		{
			;
		}
		PreviousVals = Previous->mod_values ;
	}
	
	for( i = 0; i < p_Values.size(); i ++ )
	{
		if( p_Values[ i ].length() > 0 )
		{
			NbVal ++ ;
		}
	}

	if( NbVal == 0 )
	{
		m_Mods.insert( m_Mods.end(), Mod );
		return 0;
	}
	NbVal ++ ;

	Mod->mod_values = new char *[ NbVal ];
	if( Mod->mod_values == NULL )
	{
		delete []Mod->mod_type ;
		delete Mod ;
		return -1;
	}
	/*
	* First copy the previous values
	*/
	i = 0;
	if( PreviousVals != NULL )
	{
		for( i = 0; PreviousVals[ i ] != NULL; i++ )
		{
			Mod->mod_values[ i ] = PreviousVals[ i ];
		}
		delete []PreviousVals;
	}
	long j = 0 ;
	for( ; i < p_Values.size(); i++ )
	{
		if( p_Values[ j ].length() > 0 )
		{
			wxString Value = p_Values[ j ];
//			wxString UTF8Value( wxConvUTF8.cWC2MB( Value.wc_str( *wxConvCurrent ) ) );
			Mod->mod_values[ i ] = NULL ;
			Mod->mod_values[ i ] = new char[ Value.Length() + 1 ];
			if( Mod->mod_values[ i ] != NULL )
			{
				strcpy( Mod->mod_values[ i ], Value.mb_str( wxConvUTF8 ) );
			}
		}
		j ++;
	}
	Mod->mod_values[ i ] = NULL ;
	m_Mods.insert( m_Mods.end(), Mod );
	return 0;
}

int LDAPBrowser_RightPane::UIAddAttributeValue(const wxTreeItemId &p_ParentID, const wxString &p_Value, bool p_IsNew )
{
	wxString			Name;
	StringVector	Values;
	int					iRet = LDAP_SUCCESS ;


	if( p_IsNew )
	{
		Name = GetItemText( p_ParentID );
		Values.push_back( p_Value );
		iRet = Mods_AddAttribute( LDAP_MOD_ADD, Name, Values );
	}

	if( iRet == LDAP_SUCCESS )
	{
		AppendItem( p_ParentID, p_Value );
	}
	return iRet;
}

int LDAPBrowser_RightPane::UIAddAttributeValue(const wxString &p_Name, const wxString &p_Value, bool p_IsNew)
{

	wxTreeItemId		RootID;
	wxTreeItemId		AttID;
	wxTreeItemId		FoundID;
	bool				Found = false;
	long				Cookie;
	wxString			Name;
	int					iRet = 0;

	RootID = GetRootItem();
	AttID = GetFirstChild( RootID, Cookie );
	while( AttID.IsOk() && !Found )
	{
		Name = GetItemText( AttID ).c_str();

		if( Name.CmpNoCase( p_Name ) == 0 )
		{
			FoundID = AttID;
			Found = true;
		}
		AttID = GetNextChild( RootID, Cookie );
	}
	if( !Found )
	{
		FoundID = AppendItem( RootID, p_Name );
	}

	if( FoundID.IsOk() )
	{
		iRet = UIAddAttributeValue( FoundID, p_Value, p_IsNew );
	}
	return iRet;
}

int LDAPBrowser_RightPane::AddRootItem(const wxString &p_RootDN)
{
	wxTreeItemId	RootID ;
	RootID = AddRoot( p_RootDN );
	SetItemBold( RootID );

	return 0;
}

void LDAPBrowser_RightPane::Expand()
{
	wxTreeCtrl::Expand( GetRootItem() );
}


void LDAPBrowser_RightPane::OnPopupViewBin(wxMenuEvent &p_Evt)
{
	wxTreeItemId			ParentID ;
	wxTreeItemId			RootID ;
	wxTreeItemId			CurrentSelectedItem ;
	LB_BinaryDlg			*Dlg = NULL;
	long					Pos = -1 ;

	LDAPBrowser_MainFrame	*Parent ;
	
	CurrentSelectedItem = GetSelection() ;
	RootID = GetRootItem() ;
	ParentID = GetParent( CurrentSelectedItem );


	if( ParentID == RootID || CurrentSelectedItem == RootID )
	{
		/*
		* The user must click on a item's attribute
		*/
		return;
	}

	/*
	* if the user changes the value of the binary value. The change is immediately
	* reflected
	*/

	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();

	Pos = GetAttributeIndex( CurrentSelectedItem );
	if( Pos == -1 )
	{
		return ;
	}

	berval		*BinValue = NULL ;
	wxString	AttributeName = GetItemText( ParentID );

	BinValue = Parent->m_LDAP.GetAttributeBinValue( AttributeName, Pos );
	
	if( BinValue != NULL )
	{
		Dlg = new LB_BinaryDlg( this, BinValue );
		int iRet; 
		iRet = Dlg->ShowModal();

		if( BinValue != NULL && BinValue->bv_val != NULL )
		{
			delete []BinValue->bv_val ;
			delete BinValue;
		}	
		Dlg->Destroy();
	}
	

}

void LDAPBrowser_RightPane::OnPopupDelBin( wxMenuEvent &p_Evt )
{
	wxTreeItemId			ParentID ;
	wxTreeItemId			RootID ;
	wxTreeItemId			CurrentSelectedItem ;
	long					Pos = -1 ;

	LDAPBrowser_MainFrame	*Parent ;
	
	CurrentSelectedItem = GetSelection() ;
	RootID = GetRootItem() ;
	ParentID = GetParent( CurrentSelectedItem );


	if( ParentID == RootID || CurrentSelectedItem == RootID )
	{
		/*
		* The user must click on a item's attribute
		*/
		return;
	}


	Parent = ( LDAPBrowser_MainFrame * )wxWindow::GetParent()->GetParent();

	Pos = GetAttributeIndex( CurrentSelectedItem );
	if( Pos == -1 )
	{
		return ;
	}

	berval		*BinValue = NULL ;
	int			iRet;
	BinValue = Parent->m_LDAP.GetAttributeBinValue( GetItemText( ParentID ), Pos );
	
	if( BinValue != NULL )
	{
		iRet = BinaryValueOp( LDAP_MOD_DELETE, GetItemText( ParentID ), BinValue );
		if( iRet == LDAP_SUCCESS )
		{
			BinValue = NULL ;

			iRet = OnSave();
			if( iRet == LDAP_SUCCESS )
			{
				Delete( CurrentSelectedItem );
			}
		}
		else
		{
			if( BinValue != NULL && BinValue->bv_val != NULL )
			{
				delete []BinValue->bv_val ;
				delete BinValue;
			}						
		}
	}

}
long LDAPBrowser_RightPane::GetAttributeIndex(wxTreeItemId &p_Id)
{
	wxTreeItemId	RootID ;
	wxTreeItemId	ParentID ;
	wxTreeItemId	ID;
	long			Ret = -1;
	long			i = 0;
	long			Cookie;

	
	RootID = GetRootItem();

	if( p_Id == RootID )
	{
		return Ret;
	}
	ParentID = GetParent( p_Id );

	ID = GetFirstChild( ParentID, Cookie );
	if( !ID.IsOk() )
	{
		return Ret;
	}
	i = 0;
	while( ID.IsOk() )
	{
		if( p_Id == ID )
		{
			Ret = i;
			break;
		}
		i ++;
		ID = GetNextChild( ParentID, Cookie );
	}
	return Ret;
}

void LDAPBrowser_RightPane::EmptyMods()
{
	for( unsigned long i = 0 ; i < m_Mods.size() ; i ++ )
	{
		if( m_Mods[ i ] != NULL )
		{
			if( m_Mods[ i ]->mod_type != NULL )
			{
				delete []m_Mods[ i ]->mod_type ;
			}
			if( m_Mods[ i ]->mod_values != NULL )
			{
				for( long j = 0; m_Mods[ i ]->mod_values[ j ] != NULL ; j++ )
				{
					delete []m_Mods[ i ]->mod_values[ j ] ;
				}
				delete []m_Mods[ i ]->mod_values ;
				m_Mods[ i ]->mod_values = NULL ;
			}
			if( m_Mods[ i ]->mod_bvalues != NULL )
			{
				for( long j = 0; m_Mods[ i ]->mod_bvalues[ j ] != NULL ; j++ )
				{
					if( m_Mods[ i ]->mod_bvalues[ j ]->bv_val != NULL )
					{
						delete []m_Mods[ i ]->mod_bvalues[ j ]->bv_val ;
					}
				}
				delete []m_Mods[ i ]->mod_bvalues ;
				m_Mods[ i ]->mod_bvalues = NULL ;
			}
		}
		delete m_Mods[ i ] ;
	}
	m_Mods.clear();

}
