/*! Time-stamp: <@(#)LDAP_Schema.cpp   11/02/2003 - 08:40:28   Laurent Ulrich>
 *********************************************************************
 *  @file   : LDAP_Schema.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                              Date: 11/02/2003
 *
 *  Purpose : Implementation of methods for class LDAP_Schema
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include <algorithm>

#include "../include/LDAPBrowser_Defines.h"
#include "../include/LDAP_Class.h"
#include "../include/LDAP_Schema.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LDAP_Schema::LDAP_Schema()
{
	m_IsOpen = false;
	m_LDAP = NULL;
}

LDAP_Schema::~LDAP_Schema()
{
	Close();
}

int LDAP_Schema::OpenSchema()
{
	int Ret;
	
	if( m_LDAP == NULL )
	{
		m_IsOpen = false;
		return -1;
	}

	//Ret = Search( "", "objectClass=*", 1, LDAP_SCOPE_BASE );
	Ret = Search( wxT(""), wxT("(objectclass=*)"), 1, LDAP_SCOPE_BASE );
	if( Ret != 0 || GetSize() == 0 )
	{
		m_IsOpen = false;
		return Ret;
	}

	SchemaEntry = GetAttributeValue( wxT("subschemaSubentry") );
	VendorName = GetAttributeValue( wxT("vendorName") );
	VendorVersion = GetAttributeValue( wxT("vendorVersion") );
	namingContexts = GetAttributeValues( wxT("namingContexts") );
	supportedLDAPVersion = GetAttributeValues( wxT("supportedLDAPVersion") );
	dsaName = GetAttributeValues( wxT("dsaName") );
	DirectoryTreeName = GetAttributeValues( wxT("directoryTreeName") );

	if( SchemaEntry.length() == 0 )
	{
		/*
		* Added for support of iPlanet Directory that does not have a subschema subentry field
		* in the rootDSE !!!!
		*/
		SchemaEntry = wxT("cn=schema") ;
		::wxLogMessage( wxT("RootDSE subschemaSubentry is empty: Vendor %s \n Using %s"), 
			VendorName.c_str(), SchemaEntry.c_str() );
		
	}

	Ret = Search( SchemaEntry.c_str(), wxT("(objectclass=*)"), 1, LDAP_SCOPE_BASE );
	if( Ret != 0 || GetSize() == 0 )
	{
		wxString ErrMsg ( ldap_err2string( Ret ), *wxConvCurrent );
		::wxLogMessage( wxT("Error searching schema %s\nLDAP Error: %s"), SchemaEntry.c_str(), 
			ErrMsg.c_str() );
		return false;
	}
	m_IsOpen = true;
	return 0;
}

bool LDAP_Schema::IsOpen()
{
	return m_IsOpen;
}

StringVector LDAP_Schema::GetObjectClasses()
{
	StringVector				Desc ;
	StringVector				Classes;

	long							Size ;
	long							i;
	wxString						AttributeStringValue ;
	StringVector::iterator		iter;
	StringVector				Names;

	Desc = GetAttributeValues( wxT("objectClasses") );
	Size = Desc.size();
	for( i = 0; i < Size ; i++ )
	{
		AttributeStringValue = ExtractAttribute( Desc[ i ], wxT("NAME"), wxT(" ") );
		if( AttributeStringValue.length() > 0 )
		{
			Names = GetNameValues( AttributeStringValue );
			if( Names.size() > 0 )
			{
				for( iter = Names.begin(); iter != Names.end(); iter ++ )
				{
					if( std::find( Classes.begin(), Classes.end(), ( *iter ) ) == Classes.end() )
					{
						Classes.push_back( ( *iter ) );
					}
				}
			}
		}
	}

	return Classes ;
}

StringVector LDAP_Schema::GetObjectClassAttributes( const wxString &p_ObjectClass, short p_Which )
{
	StringVector				Desc ;
	StringVector				Attributes;

	long							Size ;
	long							i;
	bool							Found = false;
	wxString						AttributeStringValue ;
	StringVector					Values;
	StringVector					Names ;
	StringVector					Sups ;
	StringVector::iterator			iter ;
	wxString						ObjectClassLine ;

	Desc = GetAttributeValues( wxT("objectClasses") );
	Size = Desc.size();
	for( i = 0; i < Size && !Found ; i++ )
	{
		
		AttributeStringValue = ExtractAttribute( Desc[ i ], wxT("NAME"), wxT(" ") );
		if( AttributeStringValue.length() > 0 )
		{
			Names = GetNameValues( AttributeStringValue );
		}
		if( Names.size() > 0 && std::find( Names.begin(), Names.end(), p_ObjectClass ) != Names.end() )
		{
			Found = true;
			ObjectClassLine = Desc[ i ] ;
		}
	}
	if( Found )
	{

		if( p_Which == ATTRIBUTE_TYPE_ALL || p_Which == ATTRIBUTE_TYPE_MUST )
		{
			AttributeStringValue = ExtractAttribute( ObjectClassLine, wxT("MUST"), wxT(")") );
			if( AttributeStringValue.length() > 0 )
			{
				Values = GetMAYMUSTValues( AttributeStringValue );
				for( iter = Values.begin(); iter != Values.end(); iter ++ )
				{
					if( std::find( Attributes.begin(), Attributes.end(), ( *iter ) ) 
						== Attributes.end() )
					{
						Attributes.push_back( (*iter) );
					}
				}
			}
		}
		if( p_Which == ATTRIBUTE_TYPE_ALL || p_Which == ATTRIBUTE_TYPE_MAY )
		{
			AttributeStringValue = ExtractAttribute( ObjectClassLine, wxT("MAY"), wxT(")") );
			if( AttributeStringValue.length() > 0 )
			{
				Values = GetMAYMUSTValues( AttributeStringValue );
				for( iter = Values.begin(); iter != Values.end(); iter ++ )
				{
					if( std::find( Attributes.begin(), Attributes.end(), ( *iter ) ) 
						== Attributes.end() )
					{
						Attributes.push_back( (*iter) );
					}
				}
			}
		}
		AttributeStringValue = ExtractAttribute( ObjectClassLine, wxT("SUP"), wxT(" ") );
		if( AttributeStringValue.length() > 0 )
		{
			Sups = GetSupValues( AttributeStringValue );
			for( unsigned long j = 0 ; j < Sups.size() ; j ++ )
			{
				Values = GetObjectClassAttributes( Sups[ j ], p_Which );
				for( iter = Values.begin(); iter != Values.end(); iter ++ )
				{
					if( std::find( Attributes.begin(), Attributes.end(), ( *iter ) ) 
						== Attributes.end() )
					{
						Attributes.push_back( (*iter) );
					}
				}
			}
		}
	}
	return Attributes;
}


wxString LDAP_Schema::ExtractAttribute( const wxString & p_Line, 
										const wxString & p_AttributeName, 
										const wxString & p_Separator )
{
	size_t Pos ;
	size_t PosEnd;

	wxString Ret;

	Pos = p_Line.find( p_AttributeName ) ;
	if( Pos == p_Line.npos )
	{
		return Ret;
	}

	Pos += p_AttributeName.length() + 1;

	while( p_Line[ Pos ] == wxT(' ') && p_Line[ Pos ] != wxT('\0') )
	{
		Pos ++;
	}
	if( p_Line[ Pos ] == wxT('\0') )
	{
		return Ret;
	}

	
	wxChar Separator ;
	switch( p_Line[ Pos ] )
	{
	case '(':
		Separator = wxT(')');
		break;
	case '\'':
		Separator = wxT('\'');
		break;
	case ' ':
		Separator = wxT(' ');
		break;
	default:
		Separator = wxT(' ');
		break;
	}
	PosEnd = p_Line.find( Separator, Pos + 1 );
	if( PosEnd == p_Line.npos )
	{
		Ret = p_Line.substr( Pos );
	}
	else
	{
		Ret = p_Line.substr( Pos, ( PosEnd - Pos ) + 1 );
	}
	

	return Ret;
}


StringVector LDAP_Schema::GetNameValues( const wxString &p_Str )
{
	StringVector		Ret;
	size_t				i;
	bool				InValue = false;
	wxString			CurrentValue ;


	for( i = 0; p_Str[ i ] != '\0'; i++ )
	{
		switch( p_Str[ i ] )
		{
		case '\'' :
			if( InValue && CurrentValue.length() > 0 )
			{
				Ret.insert( Ret.end(), CurrentValue );
				CurrentValue = wxT("");
			}
			InValue = !InValue ;
			break;
		case '(':
		case ')':
			/*
			* Ignore
			*/
			break;
		case ' ' :
		default:
			if( InValue )
			{
				CurrentValue.append( 1, p_Str[ i ] );
			}
			break;
		}
	}
	if( CurrentValue.length() > 0 )
	{
		Ret.insert( Ret.end(), CurrentValue );
	}
	return Ret;
}

StringVector LDAP_Schema::GetMAYMUSTValues(const wxString &p_Str)
{
	StringVector		Ret;
	size_t				i;
	wxString			CurrentValue ;


	for( i = 0; p_Str[ i ] != '\0'; i++ )
	{
		switch( p_Str[ i ] )
		{
		case ' ' :
			if( CurrentValue.length() > 0 )
			{
				Ret.insert( Ret.end(), CurrentValue );
				CurrentValue = wxT("");
			}
			break;
		case '$':
		case '(':
		case ')':
			/*
			* Ignore
			*/
			break;
		default:
			CurrentValue.append( 1, p_Str[ i ] );
			break;
		}
	}
	if( CurrentValue.length() > 0 )
	{
		Ret.insert( Ret.end(), CurrentValue );
	}
	return Ret;
}

StringVector LDAP_Schema::GetSupValues(const wxString &p_Str)
{
	StringVector		Ret;
	size_t				i;
	wxString			CurrentValue ;


	for( i = 0; p_Str[ i ] != '\0'; i++ )
	{
		switch( p_Str[ i ] )
		{
		case ' ' :
			if( CurrentValue.length() > 0 )
			{
				Ret.insert( Ret.end(), CurrentValue );
				CurrentValue = wxT("");
			}
			break;
		case '$':
		case '(':
		case ')':
			/*
			* Ignore
			*/
			break;
		default:
			CurrentValue.append( 1, p_Str[ i ] );
			break;
		}
	}
	if( CurrentValue.length() > 0 )
	{
		Ret.insert( Ret.end(), CurrentValue );
	}
	return Ret;
}
