/*! 
 *********************************************************************
 *  @file   : Utils.cpp
 *
 *  Project : LDAP Explorer
 *
 *  Package : LDAP
 *
 *  Company : <<>>
 *
 *  Author  : Laurent Ulrich                          Date: 19/03/2003
 *
 *  Purpose : Implementation of methods for class Utils
 *
 *********************************************************************
 */

#include <wx/wxprec.h>
#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#include "../include/LDAPBrowser_Defines.h"
#include "../include/Utils.h"

char	NOT_SAFE_INIT_CHARS[]	= "\r\n :<\0";
char	NOT_SAFE_CHARS[]		= "\r\n\0";

bool IsSafeChar( const unsigned char *p_Str, long p_Len, bool p_CheckUTF )
{

	long i = 0 ;
	unsigned char c ;
	bool Ok = true;

	while( Ok && i < p_Len )
	{
		c = p_Str[ i ] ;
		// ASCII-CHAR ?
		if( c > 127 )
		{ 
			// ! ASCII-CHAR 

			// TEST UTF-8-CHAR
			if( p_CheckUTF )
			{
				if( !( IS_UTF8_2( p_Str, i, p_Len ) ) )
					if( !( IS_UTF8_3( p_Str, i, p_Len ) ) )
						if( !( IS_UTF8_4( p_Str, i, p_Len ) ) )
							if( !( IS_UTF8_5( p_Str, i, p_Len ) ) )
								if( !( IS_UTF8_6( p_Str, i, p_Len ) ) )
								{
									Ok = false ;
								}
			}
			else
			{
				Ok = false;
			}
		}
		else
		{
			if( i == 0 )
			{
				// SAFE-INIT-CHAR ?
				if( strchr( NOT_SAFE_INIT_CHARS, c ) != NULL )
				{
					Ok = false ;
				}
			}
			else
			{
				// SAFE CHAR ?
				if( strchr( NOT_SAFE_CHARS, c ) != NULL )
				{
					Ok = false ;
				}
			}
		}
		i ++ ;
	}
	return Ok;
}


int	ToBinaryPrint( const char *p_Value, unsigned long p_Size, wxString & p_PrintableString )
{
	wxString			Buffer;
	unsigned long		i;
	
	unsigned long		maxsize ;

	p_PrintableString = wxT("");

	if( p_Size > MAX_BINARY_PRINT )
		maxsize = MAX_BINARY_PRINT ;
	else
		maxsize = p_Size ;
	p_PrintableString.reserve( maxsize * 3 + 100 );
	for( i = 0; i < maxsize ; i ++ )
	{
		Buffer.Printf( wxT("%.2X "), ( unsigned char )p_Value[ i ] );
		p_PrintableString += Buffer;
	}

	return 0 ;
}
